
# 1. Proceso con mayor participacion
create or replace PROCEDURE p_proceso_mas_participacion (RSP OUT VARCHAR2)
IS
v_fecha_proceso MESAELECTORAL.FECHAPROCESO%TYPE;
v_ambito CONVOCATORIA.AMBITO%TYPE;
BEGIN
    SELECT FECHAPROCESO, AMBITO INTO v_fecha_proceso, v_ambito FROM(
        SELECT me.FECHAPROCESO, c.AMBITO, ROUND(sum(NUMEROVOTOS) / sum(NUMEROVOTANTES) * 100,2) as participacion
        FROM MESAELECTORAL me, CONVOCATORIA c
        WHERE me.FECHAPROCESO = c.FECHAELECCIONES
        GROUP BY me.FECHAPROCESO, c.AMBITO
        ORDER BY PARTICIPACION DESC)
    WHERE ROWNUM=1;
    UPDATE h_Elecciones SET PROC_MAS_PARTIC = v_fecha_proceso || ' ' || v_ambito;
    RSP := 'OK';
    
EXCEPTION
    
    WHEN NO_DATA_FOUND THEN
    RSP := 'Error: No hay elecciones registradas';
    WHEN OTHERS THEN
    RSP := 'Error: ' || SUBSTR(SQLERRM(-SQLCODE),1,100);
END p_proceso_mas_participacion;

# 2. Municipio mayor participación
create or replace PROCEDURE p_municipio_mas_participacion (RSP OUT VARCHAR2)
IS
v_municipio MESAELECTORAL.MUNICIPIO%TYPE;
BEGIN
    SELECT MUNICIPIO INTO v_municipio FROM (
        SELECT MUNICIPIO, round (sum (NUMEROVOTOS)/sum(NUMEROVOTANTES) *100,2) as porc
        FROM MESAELECTORAL
        GROUP by municipio
        order by PORC desc)
    WHERE ROWNUM=1;
    UPDATE h_Elecciones SET MUNIC_MAS_PARTIC = v_municipio;
     RSP := 'OK';
    
    EXCEPTION
    WHEN NO_DATA_FOUND THEN
    RSP := 'Error: No hay elecciones registradas';
    WHEN OTHERS THEN
    RSP := 'Error: ' || SUBSTR(SQLERRM(-SQLCODE),1,100);
END p_municipio_mas_participacion;

# 3. Porcentaje de voto electrónico
create or replace PROCEDURE p_porcentaje_electronico_year (RSP OUT VARCHAR2)
IS
e_porcentaje EXCEPTION;
CURSOR c_anno_porc IS SELECT anno, ROUND(100*num_elect/total_votos,2) as porc FROM
                (SELECT (EXTRACT (YEAR FROM (HORAINIVOTO))) as anno, COUNT(*) AS total_votos
                 FROM VOTA 
                 group by (EXTRACT (YEAR FROM (HORAINIVOTO))))  INNER JOIN (
                SELECT (EXTRACT (YEAR FROM (HORAINIVOTO))) as anno, COUNT(*) AS num_elect
                FROM VOTA WHERE LUGARVOTO = 'Electronico' 
                group by (EXTRACT (YEAR FROM (HORAINIVOTO)))) USING(anno)
                ORDER BY porc desc;
BEGIN
    FOR porc IN c_anno_porc LOOP 
        IF porc.porc < 0 THEN
            RAISE e_porcentaje;
        END IF;
        UPDATE H_ELECCIONES SET PORC_ELECT_YEAR = porc.porc 
        WHERE EXTRACT (YEAR FROM (FECHA_ELECCIONES)) = porc.anno;
    END LOOP;
    RSP := 'OK';
    
EXCEPTION
    WHEN e_porcentaje THEN
    RSP := 'Error: No puede haber un porcentaje negativo de voto electronico';
    WHEN NO_DATA_FOUND THEN
    RSP := 'Error: No hay elecciones registradas';
    WHEN OTHERS THEN
    RSP := 'Error: ' || SUBSTR(SQLERRM(-SQLCODE),1,100);
END p_porcentaje_electronico_year;

# 4. Porcentaje de aumento/disminucion votos respecto a las anteriores elecciones

create or replace PROCEDURE p_porc_var_votos_anterior (RSP OUT VARCHAR2)
IS
CURSOR c_fecha_porc IS SELECT FECHAPROCESO, ROUND((VOTOS - LAG(VOTOS, 1, 0) 
                        OVER (ORDER BY FECHAPROCESO))/VOTOS *100,2) AS porcent_dif_votos FROM 
                        (SELECT * FROM (
                        SELECT SUM(NUMEROVOTOS) as votos, FECHAPROCESO  FROM MESAELECTORAL
                        GROUP BY FECHAPROCESO 
                        ORDER BY FECHAPROCESO)) ;
BEGIN
    FOR porc IN c_fecha_porc LOOP
        UPDATE H_ELECCIONES SET PORC_DIF_VOTO_ANT = porc.porcent_dif_votos 
        WHERE FECHA_ELECCIONES = porc.fechaproceso;
    END LOOP;
    RSP := 'OK';
    
EXCEPTION

    WHEN NO_DATA_FOUND THEN
    RSP := 'Error: No hay elecciones registradas';
    WHEN OTHERS THEN
    RSP := 'Error: ' || SUBSTR(SQLERRM(-SQLCODE),1,100);
END p_porc_var_votos_anterior;

# 5. Personas con mas de dos participaciones
create or replace PROCEDURE p_num_per_masDeDos_partc (RSP OUT VARCHAR2)
IS
v_participaciones INTEGER;
e_participaciones EXCEPTION;
BEGIN
    SELECT COUNT(*) INTO v_participaciones FROM(
        SELECT dni, COUNT(*) AS num_part FROM COMPUESTA
        GROUP BY dni)
        WHERE num_part > 2;
    IF v_participaciones < 0 THEN
    RAISE e_participaciones;
    END IF;
    IF v_participaciones > 0 THEN
    UPDATE h_Elecciones SET num_mas_dos = v_participaciones;
    END IF;
     RSP := 'OK';
    
    EXCEPTION
    WHEN e_participaciones THEN
    RSP := 'Error: El calculo del numero de participaciones no puede ser negativo';
    WHEN NO_DATA_FOUND THEN
    RSP := 'Error: No hay elecciones registradas';
    WHEN OTHERS THEN
    RSP := 'Error: ' || SUBSTR(SQLERRM(-SQLCODE),1,100);
END p_num_per_masDeDos_partc;

# 6. Porcentaje reclamaciones
create or replace PROCEDURE p_porcentaje_reclama_year (RSP OUT VARCHAR2)
IS
e_porcentaje EXCEPTION;
CURSOR c_anno_porc IS    
    SELECT anno_reclamacion, ROUND(100*num_reclamaciones/num_integrantes,2) as porc FROM (
    SELECT COUNT(*)as num_integrantes, (EXTRACT (YEAR FROM (FECHACIERRE)))AS anno_reclamacion FROM COMPUESTA
    GROUP BY EXTRACT (YEAR FROM (FECHACIERRE))) INNER JOIN (
    SELECT COUNT(*)as num_reclamaciones, (EXTRACT (YEAR FROM (FECHAALEGACION)))AS anno_reclamacion FROM COMPUESTA
    WHERE FECHAALEGACION IS NOT NULL 
    GROUP BY EXTRACT (YEAR FROM (FECHAALEGACION))) USING (anno_reclamacion);
BEGIN
    FOR rec IN c_anno_porc LOOP
        IF rec.porc < 0 THEN
            RAISE e_porcentaje;
        END IF;
        UPDATE H_ELECCIONES SET PORC_RECLAM = rec.porc 
        WHERE EXTRACT (YEAR FROM (FECHA_ELECCIONES)) = rec.anno_reclamacion;
    END LOOP;
    RSP := 'OK';
    
EXCEPTION
    WHEN e_porcentaje THEN
    RSP := 'Error: No puede haber un porcentaje negativo de reclamaciones';
    WHEN NO_DATA_FOUND THEN
    RSP := 'Error: No hay elecciones registradas';
    WHEN OTHERS THEN
    RSP := 'Error: ' || SUBSTR(SQLERRM(-SQLCODE),1,100);
END p_porcentaje_reclama_year;

# 7. Numero partidos presentados a todas las elecciones
create or replace PROCEDURE presentados_last_10 (RSP OUT VARCHAR2)
IS
v_num_part INTEGER;
e_num_pres EXCEPTION;
BEGIN
    SELECT COUNT(NUM_PRESENTACIONES) INTO v_num_part FROM(
        SELECT COUNT (ANNO_CANDIDATURA) AS num_presentaciones, siglas FROM 
        (SELECT DISTINCT (SIGLAS), (EXTRACT (YEAR FROM FECHAINICANDIDATURA))as anno_candidatura FROM PRESENTACION
            WHERE (EXTRACT (YEAR FROM FECHAINICANDIDATURA)) BETWEEN (EXTRACT (YEAR FROM (SELECT CURRENT_DATE FROM DUAL)))-10 
                                        AND (EXTRACT (YEAR FROM (SELECT CURRENT_DATE FROM DUAL)))
         ORDER BY siglas) presentados
        GROUP BY siglas)
    WHERE num_presentaciones = (SELECT COUNT(*) FROM CONVOCATORIA);
    IF v_num_part < 0 THEN
    RAISE e_num_pres;
    END IF;
    UPDATE H_ELECCIONES SET NUM_TODAS10 = v_num_part;
    RSP := 'OK';
    
EXCEPTION
    WHEN e_num_pres THEN
    RSP := 'Error: No puede haber un numero negativo de partidos presentados';
    WHEN NO_DATA_FOUND THEN
    RSP := 'Error: No existen partidos politicos';
    WHEN OTHERS THEN
    RSP := 'Error: ' || SUBSTR(SQLERRM(-SQLCODE),1,100);
END presentados_last_10;


# 8 Porcentaje poblaciones que no han realizado el sorteo (Version mejorada)
create or replace PROCEDURE p_porcentaje_poblac_sin_sorteo (RSP OUT VARCHAR2)
IS
v_por_pobl H_ELECCIONES.PORC_POBLAC_SIN_SORTEO%TYPE;

BEGIN
    SELECT round ((p.mun_sin_sort / t.total_num * 100),2) INTO v_por_pobl  FROM
        (SELECT COUNT (DISTINCT (municipio)) mun_sin_sort FROM(
            SELECT m.municipio, c.FECHASORTEO, m.MESAID, m.FECHAPROCESO, m.ambito
            FROM MESAELECTORAL m, COMPUESTA c
            WHERE m.mesaid = c.MESAID and c.FECHASORTEO IS NULL
            ORDER BY c.FECHAINISORTEO)) p,

        (SELECT COUNT (DISTINCT (municipio)) total_num FROM(
            SELECT m.municipio, c.FECHASORTEO, m.MESAID, m.FECHAPROCESO, m.ambito
            FROM MESAELECTORAL m, COMPUESTA c
            WHERE m.mesaid = c.MESAID
            ORDER BY c.FECHAINISORTEO)) t;
    UPDATE H_ELECCIONES SET PORC_POBLAC_SIN_SORTEO = v_por_pobl;
    
EXCEPTION
    WHEN NO_DATA_FOUND THEN
    RSP := 'Error: No hay elecciones registradas';
    WHEN OTHERS THEN
    RSP := 'Error: ' || SUBSTR(SQLERRM(-SQLCODE),1,100);
    
END p_porcentaje_poblac_sin_sorteo;

# 9 Top20 de mesas electorales que han recibido menos votos (presenciales o telemáticos).
create or replace PROCEDURE p_top_20_mesas (RSP OUT VARCHAR2)
AS
v_num_tablas NUMBER;
v_drop_table VARCHAR2(500);
v_tabla VARCHAR2(500);
BEGIN
    RSP := 'OK';
    SELECT COUNT(*) INTO v_num_tablas FROM user_tables WHERE table_name = 'TOP20MESAS';
    IF v_num_tablas <> 0 THEN
    v_drop_table := 'DROP TABLE TOP20MESAS';
    EXECUTE IMMEDIATE v_drop_table;
    END IF;
    v_tabla := 'CREATE TABLE TOP20MESAS AS (
                        SELECT * FROM MESAELECTORAL WHERE MESAID IN (
                        SELECT MESAID FROM(
                        SELECT ROUND(NUMEROVOTOS/NUMEROVOTANTES*100,2) PORC, MESAID FROM MESAELECTORAL
                        ORDER BY PORC ASC)
                        WHERE ROWNUM <=20))';
    EXECUTE IMMEDIATE v_tabla;
EXCEPTION
    WHEN OTHERS THEN
    RSP := 'Error: ' || SUBSTR(SQLERRM(-SQLCODE),1,100);
END p_top_20_mesas;

# 10 Porcentaje personas no localizadas
create or replace PROCEDURE p_porcentaje_sin_localizar (RSP OUT VARCHAR2)
IS
CURSOR c_porc_ambito IS
    SELECT ROUND((na.num_aleg / np.num_per * 100),2) as porc, na.ambito, na.fechaproceso
    FROM 
        (SELECT COUNT(*) as num_aleg, me.fechaproceso, me.ambito
        FROM MESAELECTORAL me, COMPUESTA c
        WHERE me.MESAID = c.MESAID AND c.FECHANOTIF IS NOT NULL AND me.fechaproceso = (SELECT MAX(FECHAPROCESO) FROM MESAELECTORAL)
        GROUP BY me.fechaproceso, me.ambito) na,
        (SELECT COUNT(*) as num_per, me.fechaproceso, me.ambito
        FROM MESAELECTORAL me, COMPUESTA c
        WHERE me.MESAID = c.MESAID AND  me.fechaproceso = (SELECT MAX(FECHAPROCESO) FROM MESAELECTORAL)
        GROUP BY me.fechaproceso, me.ambito) np
WHERE na.ambito = np.ambito;
BEGIN
    RSP := 'OK';
    FOR amb IN c_porc_ambito LOOP
        UPDATE H_ELECCIONES SET PORC_SIN_LOC = amb.porc 
        WHERE FECHA_ELECCIONES = amb.fechaproceso AND AMBITO = amb.ambito;
    END LOOP;
EXCEPTION
    WHEN OTHERS THEN
    RSP := 'Error: ' || SUBSTR(SQLERRM(-SQLCODE),1,100);
END p_porcentaje_sin_localizar;
    


# 11 proceso electoral donde ha habido más candidatos
create or replace PROCEDURE p_proceso_mas_candidatos (RSP OUT VARCHAR2)
IS
v_fecha_proceso CONVOCATORIA.FECHAELECCIONES%TYPE;
v_ambito CONVOCATORIA.AMBITO%TYPE;
BEGIN
    SELECT FECHAELECCIONES, AMBITO INTO v_fecha_proceso, v_ambito FROM(
        SELECT COUNT(*) num_pres, c.FECHACENSO, c.AMBITO, c.FECHAELECCIONES FROM PRESENTACION p, CONVOCATORIA c
        WHERE p.FECHACIERRE = c.FECHACENSO
        GROUP BY c.FECHACENSO, c.FECHAELECCIONES, c.AMBITO
        ORDER BY num_pres DESC)
    WHERE ROWNUM=1;
    
    UPDATE h_Elecciones SET PROC_MAS_CANDID = v_fecha_proceso || ' ' || v_ambito;
    RSP := 'OK';
    
    EXCEPTION
    WHEN NO_DATA_FOUND THEN
    RSP := 'Error: No hay elecciones registradas';
    WHEN OTHERS THEN
    RSP := 'Error: ' || SUBSTR(SQLERRM(-SQLCODE),1,100);
END p_proceso_mas_candidatos;

# 12 Resultado de las elecciones.

create or replace PROCEDURE p_cal_res_ult_elec (RSP OUT VARCHAR2)
IS
CURSOR c_fecha_ambito IS SELECT  DISTINCT (ambito), fechaproceso
                        FROM MESAELECTORAL 
                        WHERE FECHAPROCESO = ( SELECT MAX(fechaproceso) FROM MESAELECTORAL); 
BEGIN
    FOR fa IN c_fecha_ambito LOOP
        p_tabla_temp_res(fa.fechaproceso, fa.ambito, RSP);
        p_res_demarc(fa.fechaproceso, fa.ambito,  RSP);
        p_num_escannos(fa.ambito, RSP);
    END LOOP;
    RSP := 'OK';
EXCEPTION
    WHEN NO_DATA_FOUND THEN
    RSP := 'Error: No hay datos sobre fecha y ambito de elecciones';
    WHEN OTHERS THEN
    RSP := 'Error: ' || SUBSTR(SQLERRM(-SQLCODE),1,100);
END p_cal_res_ult_elec;

----------------------------------------------------------------------------------------------------------

create or replace PROCEDURE p_tabla_temp_res (p_fecha IN DATE, p_ambito IN VARCHAR2, RSP OUT VARCHAR2 )
IS

CURSOR c_datos_res IS SELECT * from 
        (SELECT SUM(nv.num_votos) as sum_votos, nv.siglas, nv.DEMARCACION, nv.fechaproceso, nv.ambito, 
        (SUM(nv.num_votos) * 0) as num_esc from (
            SELECT COUNT(rv.HORARECUENTO) as num_votos, rv.SIGLAS, rv.MESAID, me.FECHAPROCESO, me.AMBITO, me.DEMARCACION
            FROM RECIBEVOTO rv, MESAELECTORAL me
            WHERE rv.MESAID = me.MESAID AND rv.ESNULO = 0 AND rv.ESBLANCO = 0 
            GROUP BY rv.SIGLAS, me.DEMARCACION, rv.MESAID, me.fechaproceso, me.ambito)  nv
        GROUP BY nv.SIGLAS, nv.DEMARCACION, nv.fechaproceso, nv.ambito) v
    WHERE v.fechaproceso = p_fecha AND v.ambito = p_ambito;

e_fecha EXCEPTION;
e_ambito EXCEPTION;
BEGIN 
    IF p_fecha IS NULL OR p_fecha = '' THEN
        RAISE e_fecha;
    ELSIF p_ambito IS NULL OR p_ambito = '' THEN
        RAISE e_ambito;
    END IF;
    
    DELETE FROM TEMP_TABLE_RES;
    FOR d_res IN c_datos_res LOOP
        INSERT INTO TEMP_TABLE_RES VALUES (d_res.SUM_VOTOS, d_res.SIGLAS, d_res.DEMARCACION, d_res.NUM_ESC);
    END LOOP;
    --p_res_demarc(p_ambito, p_fecha,  RSP);
    --p_num_escannos(p_ambito, RSP);
    RSP := 'OK';
EXCEPTION

    WHEN e_fecha THEN
    RSP := 'Error: La fecha de las elecciones no puede ser nula';
    WHEN e_ambito THEN
    RSP := 'Error: El ambito de las elecciones no puede ser nulo';
    WHEN NO_DATA_FOUND THEN
    RSP := 'Error: No hay elecciones registradas';
    WHEN OTHERS THEN
    RSP := 'Error: ' || SUBSTR(SQLERRM(-SQLCODE),1,100);
END p_tabla_temp_res;

--------------------------------------------------------------------------------------------------

create or replace PROCEDURE p_res_demarc (p_fecha_elec IN DATE, p_ambito IN RECIBE.AMBITO%TYPE, RSP OUT VARCHAR2)
AS
CURSOR v_demarcaciones IS SELECT DISTINCT DEMARCACION FROM temp_table_res;
num_esc_prov INTEGER;
sigla_mas_votada temp_table_res.SIGLAS%TYPE;
v_ambito VARCHAR (20 CHAR);

e_ambito EXCEPTION;
BEGIN 
    IF p_ambito IS NULL OR p_ambito = '' THEN
        RAISE e_ambito;
    END IF;
    v_ambito := p_ambito;
    FOR c_demarcacion IN v_demarcaciones LOOP
        SELECT NUMESCANOS INTO num_esc_prov 
        FROM RECIBE r
        WHERE r.PROVINCIA = c_demarcacion.demarcacion AND r.AMBITO = p_ambito AND r.FECHACONVBOE = 
        (SELECT c.FECHACONVBOE FROM CONVOCATORIA c WHERE c.FECHAELECCIONES=p_fecha_elec);
        FOR i in 1..num_esc_prov LOOP
            sigla_mas_votada:= max_result(c_demarcacion.DEMARCACION);
            p_update_temp_table_res(c_demarcacion.DEMARCACION, sigla_mas_votada, RSP);
        END LOOP;
    END LOOP;
    RSP := 'OK';
    
EXCEPTION
    WHEN e_ambito THEN
    RSP := 'Error: Valor null del parametro ambito';
    WHEN OTHERS THEN
    RSP := 'Error: ' || SUBSTR(SQLERRM(-SQLCODE),1,100);
END p_res_demarc;

-----------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION max_result (prov IN temp_table_res.DEMARCACION%TYPE) 
RETURN temp_table_res.SIGLAS%TYPE
IS 
v_max temp_table_res.SIGLAS%TYPE;
BEGIN
SELECT SIGLAS INTO v_max FROM (
    SELECT * from temp_table_res
    WHERE DEMARCACION = prov
    ORDER BY temp_table_res.VOTOS_OBTENIDOS desc)
WHERE ROWNUM=1;
RETURN v_max;
END;

----------------------------------------------------------------------------------------------------------------------------------

create or replace PROCEDURE p_update_temp_table_res (p_prov IN temp_table_res.DEMARCACION%TYPE, p_sig IN temp_table_res.SIGLAS%TYPE, 
                                                        RSP OUT VARCHAR2)
IS
v_numEsc INTEGER;
e_demarcacion EXCEPTION;
e_siglas EXCEPTION;
BEGIN 
    IF p_prov IS NULL THEN
        RAISE e_demarcacion;
    ELSIF p_sig IS NULL THEN
        RAISE e_siglas;
    END IF;
    SELECT num_esc INTO v_numEsc FROM temp_table_res where DEMARCACION = p_prov and SIGLAS = p_sig;
    UPDATE temp_table_res SET
        num_esc = v_numEsc + 1,
        SUM_VOTOS = SUM_VOTOS / (v_numEsc + 1)
    WHERE DEMARCACION = p_prov AND SIGLAS = p_sig;
    RSP := 'OK';
    
EXCEPTION
    WHEN e_demarcacion THEN
    RSP := 'Error: Valor proporcionado null, la demarcación debe ser valida';
    WHEN e_siglas THEN
    RSP := 'Error: Valor proporcionado null, las siglas de un partido deben ser validas';
    WHEN NO_DATA_FOUND THEN
    RSP := 'Error: No hay datos sobre el numero de escaños';
    WHEN OTHERS THEN
    RSP := 'Error: ' || SUBSTR(SQLERRM(-SQLCODE),1,100);
END p_update_temp_table_res;


--------------------------------------------------------------------------------------


create or replace PROCEDURE p_num_escannos (p_ambito IN VARCHAR2, RSP OUT VARCHAR2)
AS
v_num_tablas NUMBER;
v_drop_table VARCHAR2(500);
v_tabla VARCHAR2(500);
e_ambito EXCEPTION;
BEGIN
    IF p_ambito IS NULL OR p_ambito = '' THEN
        RAISE e_ambito;
    END IF;
    
    SELECT COUNT(*) INTO v_num_tablas FROM user_tables WHERE table_name = 'NUMESCGEN';
    IF v_num_tablas <> 0 THEN
    v_drop_table := 'DROP TABLE NUMESCGEN';
    EXECUTE IMMEDIATE v_drop_table;
    END IF;
    v_tabla := 'CREATE TABLE NUMESCGEN AS (SELECT * FROM (SELECT SIGLAS, SUM(NUM_ESC) AS TOTAL_ESC  FROM TEMP_TABLE_RES 
                                                    GROUP BY SIGLAS ORDER BY SUM(NUM_ESC) DESC))';
    EXECUTE IMMEDIATE v_tabla;
    RSP := 'OK';
EXCEPTION
    WHEN e_ambito THEN
    RSP := 'Error: Valor null del parametro ambito';
    WHEN OTHERS THEN
    RSP := 'Error: ' || SUBSTR(SQLERRM(-SQLCODE),1,100);
END p_num_escannos;


---------------------------------------------CARGA INICIAL DE TODOS LOS PROCESOS-------------------------------------------------
create or replace PROCEDURE p_set_procesos (RSP OUT VARCHAR2)
IS
v_fecha_elec H_ELECCIONES.FECHA_ELECCIONES%TYPE;
v_ambito H_ELECCIONES.AMBITO%TYPE;
CURSOR c_elecciones  IS (SELECT FECHAELECCIONES, AMBITO FROM CONVOCATORIA);
BEGIN
    FOR eleccion IN c_elecciones LOOP 
        v_fecha_elec := eleccion.FECHAELECCIONES;
        v_ambito := eleccion.ambito;
        INSERT INTO H_ELECCIONES(FECHA_ELECCIONES,AMBITO) VALUES (eleccion.FECHAELECCIONES, eleccion.ambito);
    END LOOP;
    RSP := 'OK';
    
EXCEPTION
    WHEN dup_val_on_index THEN
    RSP := 'Ya existen unas elecciones ' || v_ambito || ' el ' || v_fecha_elec || 
        '. Es necesario comprobar el resto de inserciones';
    WHEN NO_DATA_FOUND THEN
    RSP := 'Error: No hay elecciones registradas';
    WHEN OTHERS THEN
    RSP := 'Error: ' || SUBSTR(SQLERRM(-SQLCODE),1,100);
END;

-----------------------------------DROOP-------------------------------------------------------
create or replace PROCEDURE p_res_droop (RSP OUT VARCHAR2)
IS
CURSOR c_fecha_ambito IS SELECT  DISTINCT (ambito), fechaproceso
                        FROM MESAELECTORAL 
                        WHERE FECHAPROCESO = ( SELECT MAX(fechaproceso) FROM MESAELECTORAL); 
BEGIN
    FOR fa IN c_fecha_ambito LOOP
        p_tabla_temp_res(fa.fechaproceso, fa.ambito, RSP);
        p_upd_table_droop(fa.fechaproceso, fa.ambito, RSP);
        p_num_escannos(fa.ambito, RSP);
    END LOOP;
    RSP := 'OK';
EXCEPTION
    WHEN NO_DATA_FOUND THEN
    RSP := 'Error: No hay datos sobre fecha y ambito de elecciones';
    WHEN OTHERS THEN
    RSP := 'Error: ' || SUBSTR(SQLERRM(-SQLCODE),1,100);
END p_res_droop;

---------------------------------------------------------------------------

create or replace PROCEDURE p_upd_table_droop (p_fecha_elec IN DATE, p_ambito IN RECIBE.AMBITO%TYPE, RSP OUT VARCHAR2)
AS
num_esc_prov INTEGER;
num_votos_prov INTEGER;
v_coc_droop INTEGER;
v_esc_asig INTEGER;
v_dif_esc INTEGER;

CURSOR c_dems IS SELECT DISTINCT DEMARCACION FROM temp_table_res;
CURSOR c_siglas IS SELECT DISTINCT SIGLAS FROM temp_table_res;

e_ambito EXCEPTION;

BEGIN 
    IF p_ambito IS NULL OR p_ambito = '' THEN
        RAISE e_ambito;
    END IF;

    FOR c_dem IN c_dems LOOP
        SELECT NUMESCANOS INTO num_esc_prov 
        FROM RECIBE r
        WHERE r.PROVINCIA = c_dem.demarcacion AND r.AMBITO = p_ambito AND r.FECHACONVBOE = 
        (SELECT c.FECHACONVBOE FROM CONVOCATORIA c WHERE c.FECHAELECCIONES=p_fecha_elec);

        SELECT SUM(SUM_VOTOS) INTO num_votos_prov FROM temp_table_res WHERE DEMARCACION= c_dem.demarcacion;

        SELECT CAST(( 1+(num_votos_prov / (num_esc_prov + 1))) AS INTEGER) INTO v_coc_droop FROM DUAL;
        
        FOR c_sig IN c_siglas LOOP
        
            UPDATE TEMP_TABLE_RES SET NUM_ESC = FLOOR (SUM_VOTOS / v_coc_droop), 
                                  SUM_VOTOS = MOD (SUM_VOTOS, v_coc_droop)
            WHERE SIGLAS = c_sig.siglas AND DEMARCACION = c_dem.demarcacion;
        END LOOP;
        SELECT SUM(NUM_ESC) INTO v_esc_asig FROM TEMP_TABLE_RES WHERE DEMARCACION = c_dem.demarcacion;
        v_dif_esc := num_esc_prov - v_esc_asig;
        IF v_dif_esc > 0 THEN
           FOR i IN 1..v_dif_esc LOOP
                UPDATE TEMP_TABLE_RES SET NUM_ESC = NUM_ESC + 1, SUM_VOTOS = 0
                WHERE DEMARCACION = c_dem.demarcacion AND SIGLAS = (SELECT t.SIGLAS FROM TEMP_TABLE_RES t 
                            WHERE DEMARCACION = c_dem.demarcacion AND SUM_VOTOS =(SELECT MAX(SUM_VOTOS) 
                            FROM TEMP_TABLE_RES WHERE DEMARCACION = c_dem.demarcacion));
            END LOOP;
        END IF;
    END LOOP;
    RSP := 'OK';
    
EXCEPTION
    WHEN e_ambito THEN
    RSP := 'Error: Valor null del parametro ambito';
    WHEN OTHERS THEN
    RSP := 'Error: ' || SUBSTR(SQLERRM(-SQLCODE),1,100);
END p_upd_table_droop;

--------------------------------MEJORAS DE LAS ESTADÍSTICAS------------------------------------------------

ALTER TABLE H_ELECCIONES ADD PORC_PARTICIP NUMBER DEFAULT (0);
ALTER TABLE H_ELECCIONES ADD PORC_NULO NUMBER DEFAULT (0);
ALTER TABLE H_ELECCIONES ADD PORC_BLANCO NUMBER DEFAULT (0);


---------------------PORCENTAJE DE PARTICIPACIÓN----------------------------------------------

create or replace PROCEDURE p_porc_participacion (RSP OUT VARCHAR2)
IS
CURSOR c_porc_fecha_amb IS SELECT ROUND(VOTANTES / CENSADOS * 100,2) porc_part, v.FECHAPROCESO, v.ambito FROM (   
        SELECT COUNT(*) as censados, FECHACIERRE from persona GROUP BY FECHACIERRE)c ,
        (SELECT COUNT(*)AS VOTANTES, m.FECHAPROCESO, VOTA.FECHACIERRE, m.ambito
        FROM VOTA, MESAELECTORAL m 
        WHERE VOTA.MESAID = m.mesaid
        GROUP BY m.FECHAPROCESO, VOTA.FECHACIERRE, m.ambito) v
    WHERE c.FECHACIERRE = v.FECHACIERRE;

e_porcentaje EXCEPTION;
BEGIN
    FOR porc IN c_porc_fecha_amb LOOP 
        IF porc.porc_part < 0 THEN
            RAISE e_porcentaje;
        END IF;
        UPDATE H_ELECCIONES SET PORC_PARTICIP = porc.porc_part 
        WHERE  FECHA_ELECCIONES = porc.FECHAPROCESO AND AMBITO = porc.ambito;
    END LOOP;
    RSP := 'OK';

EXCEPTION
    WHEN e_porcentaje THEN
    RSP := 'Error: No puede haber un porcentaje negativo de participación';
    WHEN NO_DATA_FOUND THEN
    RSP := 'Error: No hay elecciones registradas';
    WHEN OTHERS THEN
    RSP := 'Error: ' || SUBSTR(SQLERRM(-SQLCODE),1,100);
END p_porc_participacion;

-----------------------------PORCENTAJE DE VOTOS NULOS------------------------------------------

create or replace PROCEDURE p_porc_nulos (RSP OUT VARCHAR2)
IS
CURSOR c_porc_fecha_amb IS SELECT DISTINCT ROUND(nl.num_nulos / nt.num_total * 100,2) as porc_nulos, me.AMBITO, 
                                            me.FECHAPROCESO FROM (
    SELECT COUNT(*) num_nulos FROM RECIBEVOTO
    WHERE ESNULO = 1 AND HORAINIRECUENTO = (SELECT MAX( HORAINIRECUENTO) FROM RECIBEVOTO)) nl,
    (SELECT COUNT(*) num_total FROM RECIBEVOTO 
    WHERE HORAINIRECUENTO = (SELECT MAX(HORAINIRECUENTO) FROM RECIBEVOTO)) nt,
    (SELECT FECHAPROCESO, AMBITO FROM MESAELECTORAL 
    WHERE FECHAPROCESO = (SELECT MAX(FECHAPROCESO) FROM MESAELECTORAL)) me
;

e_porcentaje EXCEPTION;
BEGIN
    FOR porc IN c_porc_fecha_amb LOOP 
        IF porc.porc_nulos < 0 THEN
            RAISE e_porcentaje;
        END IF;
        UPDATE H_ELECCIONES SET PORC_NULO = porc.porc_nulos 
        WHERE  FECHA_ELECCIONES = porc.FECHAPROCESO AND AMBITO = porc.AMBITO;
    END LOOP;
    RSP := 'OK';

EXCEPTION
    WHEN e_porcentaje THEN
    RSP := 'Error: No puede haber un porcentaje negativo de votos nulos';
    WHEN NO_DATA_FOUND THEN
    RSP := 'Error: No hay elecciones registradas';
    WHEN OTHERS THEN
    RSP := 'Error: ' || SUBSTR(SQLERRM(-SQLCODE),1,100);
END p_porc_nulos;

-----------------------------PORCENTAJE DE VOTOS EN BLANCO -------------------------------------
create or replace PROCEDURE p_porc_blancos (RSP OUT VARCHAR2)
IS
CURSOR c_porc_fecha_amb IS SELECT DISTINCT ROUND(nl.num_nulos / nt.num_total * 100,2) as porc_blancos, me.AMBITO, 
                                            me.FECHAPROCESO FROM (
    SELECT COUNT(*) num_nulos FROM RECIBEVOTO
    WHERE ESNULO = 0 AND ESBLANCO = 1 AND HORAINIRECUENTO = (SELECT MAX( HORAINIRECUENTO) FROM RECIBEVOTO)) nl,
    (SELECT COUNT(*) num_total FROM RECIBEVOTO 
    WHERE HORAINIRECUENTO = (SELECT MAX(HORAINIRECUENTO) FROM RECIBEVOTO)) nt,
    (SELECT FECHAPROCESO, AMBITO FROM MESAELECTORAL 
    WHERE FECHAPROCESO = (SELECT MAX(FECHAPROCESO) FROM MESAELECTORAL)) me
;

e_porcentaje EXCEPTION;
BEGIN
    FOR porc IN c_porc_fecha_amb LOOP 
        IF porc.porc_blancos < 0 THEN
            RAISE e_porcentaje;
        END IF;
        UPDATE H_ELECCIONES SET PORC_BLANCO = porc.porc_blancos 
        WHERE  FECHA_ELECCIONES = porc.FECHAPROCESO AND AMBITO = porc.AMBITO;
    END LOOP;
    RSP := 'OK';

EXCEPTION
    WHEN e_porcentaje THEN
    RSP := 'Error: No puede haber un porcentaje negativo de votos nulos';
    WHEN NO_DATA_FOUND THEN
    RSP := 'Error: No hay elecciones registradas';
    WHEN OTHERS THEN
    RSP := 'Error: ' || SUBSTR(SQLERRM(-SQLCODE),1,100);
END p_porc_blancos;

----------------------------Porcentaje de voto por sexo-------------------------------------------

create or replace PROCEDURE p_estd_por_sexo (RSP OUT VARCHAR2)
AS
v_num_tablas NUMBER;
v_drop_table VARCHAR2(500);
v_tabla VARCHAR2(500);
BEGIN
    RSP := 'OK';
    SELECT COUNT(*) INTO v_num_tablas FROM user_tables WHERE table_name = 'ESTSEXO';
    IF v_num_tablas <> 0 THEN
    v_drop_table := 'DROP TABLE ESTSEXO';
    EXECUTE IMMEDIATE v_drop_table;
    END IF;
    v_tabla := 'CREATE TABLE ESTSEXO AS (
                        SELECT vs.PORC_VOTOS, presis.PORC_PRESIDENTES, pcand.PORC_CANDIDATOS, vs.sexo from 
                        (SELECT ROUND (ps.n_votos / t.vot_totales * 100,2) PORC_VOTOS, ps.SEXO, t.FECHA_VOTO FROM 

                        (SELECT COUNT(*) vot_totales, CAST(HORAINIVOTO AS DATE) AS FECHA_VOTO 
                        FROM VOTA WHERE HORAINIVOTO = (SELECT MAX(v.HORAINIVOTO) FROM VOTA v) 
                        GROUP BY HORAINIVOTO) t,

                        (SELECT COUNT(DNI) n_votos, SEXO, FECHACIERRE 
                        from persona 
                        WHERE DNI IN (SELECT DNI FROM VOTA WHERE HORAINIVOTO = (SELECT MAX(v.HORAINIVOTO) FROM VOTA v))
                        AND FECHACIERRE = (SELECT MAX(p.FECHACIERRE) FROM PERSONA p)
                        GROUP BY SEXO, FECHACIERRE) ps) vs,


                        (SELECT ROUND (pps.n_votos / tp.vot_totales * 100,2) PORC_PRESIDENTES, pps.SEXO, tp.FECHACIERRE FROM

                        (SELECT COUNT(DNI) n_votos, SEXO, FECHACIERRE 
                        from persona 
                        WHERE DNI IN (SELECT DNI FROM PRESIDENTE WHERE FECHACIERRE = (SELECT MAX(v.FECHACIERRE) FROM PRESIDENTE v))
                        AND FECHACIERRE = (SELECT MAX(p.FECHACIERRE) FROM PERSONA p)
                        GROUP BY SEXO, FECHACIERRE) pps,

                        (SELECT COUNT(*) vot_totales, FECHACIERRE 
                        FROM PRESIDENTE WHERE FECHACIERRE = (SELECT MAX(v.FECHACIERRE) FROM PRESIDENTE v) 
                        GROUP BY FECHACIERRE) tp) presis,

                        (SELECT round (cps.n_votos / tpres.vot_totales * 100, 2) AS PORC_CANDIDATOS, cps.sexo, cps.fechacierre FROM 
                        (SELECT COUNT(DNI) n_votos, SEXO, FECHACIERRE 
                        from persona 
                        WHERE DNI IN (SELECT DNI FROM PRESENTACION WHERE FECHACIERRE = (SELECT MAX(v.FECHACIERRE) FROM PRESENTACION v))
                        AND FECHACIERRE = (SELECT MAX(p.FECHACIERRE) FROM PERSONA p)
                        GROUP BY SEXO, FECHACIERRE) cps,

                        (SELECT COUNT(*) vot_totales, FECHACIERRE 
                        FROM PRESENTACION WHERE FECHACIERRE = (SELECT MAX(v.FECHACIERRE) FROM PRESENTACION v) 
                        GROUP BY FECHACIERRE) tpres) pcand
                        WHERE pcand.SEXO = presis.sexo AND presis.sexo = vs.sexo;';
    EXECUTE IMMEDIATE v_tabla;
EXCEPTION
    WHEN OTHERS THEN
    RSP := 'Error: ' || SUBSTR(SQLERRM(-SQLCODE),1,100);
END p_estd_por_sexo;