-- PROCEDIMIENTOS PARA REALIZAR LOS TEST DE INSERT, UPDATE y DELETE EN LAS TABLAS --


-- Procedimiento para el test de la tabla convocatoria 

create or replace PROCEDURE p_test_conv (
    p_fec_conv IN convocatoria.fechaConvBOE%TYPE, 
    p_fec_elec IN convocatoria.fechaElecciones%TYPE,
    p_ambito IN CONVOCATORIA.AMBITO%TYPE,
    p_fec_censo IN CONVOCATORIA.FECHACENSO%TYPE,
    p_regla IN CONVOCATORIA.REGLACALCRESULTADO%TYPE,
    p_tabla_mod IN VARCHAR2,
    p_tipo_oper IN VARCHAR2,
    RSP OUT VARCHAR2)
AS
    v_proced TEST_LOG.PROCEDIMIENTO%TYPE := 'p_test_conv';

    e_null_fec_conv EXCEPTION;
    e_null_ambito EXCEPTION;
    e_null_fecha_elec EXCEPTION;
    e_enum_ambito EXCEPTION;
    e_null_regla EXCEPTION;
    e_enum_reglas EXCEPTION;
    e_plazo EXCEPTION;
    e_fk EXCEPTION;
    pragma exception_init(e_fk, -2292);
    
BEGIN
    -- Inicio del procedimiento

    -- Comprobación de las restricciones de la tabla y llamadas a las excepciones.
    IF (p_fec_conv IS NULL OR p_fec_conv = '') THEN
        RAISE e_null_fec_conv;
    ELSIF (p_ambito IS NULL OR p_ambito ='') THEN
        RAISE e_null_ambito;
    ELSIF (p_fec_elec IS NULL OR p_fec_elec = '') THEN
        RAISE e_null_fecha_elec;
    ELSIF (p_ambito NOT IN ('Generales', 'Municipales', 'Parlamento Europeo', 'Autonomicas')) THEN
        RAISE e_enum_ambito;
    ELSIF (p_regla IS NULL OR p_regla ='') THEN
        RAISE e_null_regla;
    ELSIF (p_regla NOT IN ('D’Hondt', 'Droop', 'Hagenbach-Bischoff', 'Resto Mayor', 'Sainte-Laguë')) THEN
        RAISE e_enum_reglas;
    ELSIF (p_fec_elec < p_fec_conv + 45) THEN
        RAISE e_plazo;
    ELSIF SQLCODE = 02292 THEN
        RAISE e_fk;
    END IF;
    
    -- Asignación del valor OK a la variable de salida
    -- Selección de la operación a realizar 
    CASE 
    WHEN p_tipo_oper = 'INSERT' THEN
        -- Inserción en la tabla.
        INSERT INTO CONVOCATORIA VALUES (p_fec_conv, p_fec_elec, p_ambito, p_fec_censo, p_regla);
        RSP := 'OK';
        -- Insertar el resultado en la tabla test_log
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    WHEN p_tipo_oper = 'UPDATE' THEN
        RSP := 'OK';
        UPDATE CONVOCATORIA SET AMBITO = p_ambito WHERE FECHACONVBOE = p_fec_conv;
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    WHEN p_tipo_oper = 'DELETE' THEN
        RSP := 'OK';
        DELETE FROM CONVOCATORIA WHERE FECHACONVBOE = p_fec_conv;
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    END CASE;
    
    -- Tratamiento de excepciones
    EXCEPTION
        WHEN e_null_fec_conv THEN
            RSP := 'ERROR: Fecha convocatoria nula';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_ambito THEN
            RSP := 'ERROR: Ambito de elecciones nulo';  
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
             VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_fecha_elec THEN
            RSP := 'ERROR: Fecha de elecciones nula';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_enum_ambito THEN
            RSP := 'ERROR: Ambito de elecciones no válido';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_regla THEN
            RSP := 'ERROR: Regla de calculo de resultado nula';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_enum_reglas THEN
            RSP := 'ERROR: Regla de calculo de resultado no válida';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_plazo THEN
            RSP := 'ERROR: El plazo debe ser mayor de 44 dias';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_fk THEN
            RSP := 'Error: Existe una clave foranea que referencia a Convocatoria';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN DUP_VAL_ON_INDEX THEN
            RSP := 'ERROR: Ya existe una convocatoria con esta PK'; 
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN NO_DATA_FOUND THEN
            RSP := 'ERROR: No existen datos con los parametros dados';  
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN OTHERS THEN
            RSP := 'ERROR: ' || SUBSTR(SQLERRM(-SQLCODE),1,100); 
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);

        END p_test_conv;

-- Procedimiento para el test de la tabla ReglasElecciones

create or replace PROCEDURE p_test_reglas_elec (
    p_fec_conv IN ReglasElecciones.fechaConvBOE%TYPE, 
    p_ambito IN ReglasElecciones.AMBITO%TYPE,
    p_num_regla IN REGLASELECCIONES.NREGLA%TYPE,
    p_regla IN REGLASELECCIONES.REGLA%TYPE,
    p_tabla_mod IN VARCHAR2,
    p_tipo_oper IN VARCHAR2,
    RSP OUT VARCHAR2)
AS
    v_proced TEST_LOG.PROCEDIMIENTO%TYPE := 'p_test_reglas_elec';

    e_null_fec_conv EXCEPTION;
    e_null_ambito EXCEPTION;
    e_enum_ambito EXCEPTION;
    e_null_n_regla EXCEPTION;
    e_no_pk_ref EXCEPTION;
    pragma exception_init(e_no_pk_ref, -2291);
    e_fk EXCEPTION;
    pragma exception_init(e_fk, -2292);

BEGIN
    -- Inicio del procedimiento

    -- Comprobación de parámetros
    IF (p_fec_conv IS NULL OR p_fec_conv = '') THEN
        RAISE e_null_fec_conv;
    ELSIF (p_ambito IS NULL OR p_ambito ='') THEN
        RAISE e_null_ambito;
    ELSIF (p_ambito NOT IN ('Generales', 'Municipales', 'Parlamento Europeo', 'Autonomicas')) THEN
        RAISE e_enum_ambito;
    ELSIF (p_num_regla IS NULL OR p_num_regla ='') THEN
        RAISE e_null_n_regla;
    END IF;

    -- Asignación del valor OK a la variable de salida
    -- Selección de la operación a realizar 
    CASE 
    WHEN p_tipo_oper = 'INSERT' THEN
        -- Inserción en la tabla.
        INSERT INTO ReglasElecciones VALUES (p_fec_conv, p_ambito, p_num_regla, p_regla);
        RSP := 'OK';
        -- Insertar el resultado en la tabla test_log
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    WHEN p_tipo_oper = 'UPDATE' THEN
        RSP := 'OK';
        UPDATE ReglasElecciones SET AMBITO = p_ambito WHERE FECHACONVBOE = p_fec_conv;
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    WHEN p_tipo_oper = 'DELETE' THEN
        RSP := 'OK';
        DELETE FROM ReglasElecciones WHERE NREGLA = p_num_regla;
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    END CASE;

    -- Tratamiento de excepciones
    EXCEPTION
        WHEN e_null_fec_conv THEN
            RSP := 'ERROR: Fecha Convocatoria nula';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_ambito THEN
            RSP := 'ERROR: Ambito de elecciones nulo';  
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
             VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_n_regla THEN
            RSP := 'ERROR: Numero de regla nulo';   
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
             VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_enum_ambito THEN
            RSP := 'ERROR: Ambito de elecciones no válido';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_no_pk_ref THEN
            RSP := 'Error: No existe la clave principal referenciada';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_fk THEN
            RSP := 'Error: Existe una clave foranea que referencia a ReglasElecciones';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN DUP_VAL_ON_INDEX THEN
            RSP := 'ERROR: Ya existe una ReglasElecciones con esta PK'; 
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN NO_DATA_FOUND THEN
            RSP := 'ERROR: No existen datos con los parametros dados';  
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN OTHERS THEN
            RSP := 'ERROR: ' || SUBSTR(SQLERRM(-SQLCODE),1,100); 
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);

END p_test_reglas_elec;


-- Procedimiento para el test de la tabla Demarcacion
create or replace PROCEDURE p_test_demarcacion (
    p_provincia IN Demarcacion.PROVINCIA%TYPE, 
    p_region IN Demarcacion.REGION%TYPE,
    p_pais IN Demarcacion.PAIS%TYPE,
    p_tabla_mod IN VARCHAR2,
    p_tipo_oper IN VARCHAR2,
    RSP OUT VARCHAR2)
AS
    v_proced TEST_LOG.PROCEDIMIENTO%TYPE := 'p_test_demarcacion';

    e_null_prov EXCEPTION;
    e_null_region EXCEPTION;
    e_null_pais EXCEPTION;
    e_no_pk_ref EXCEPTION;
    pragma exception_init(e_no_pk_ref, -2291);
    e_fk EXCEPTION;
    pragma exception_init(e_fk, -2292);

BEGIN
    -- Inicio del procedimiento

    -- Comprobación de parámetros
    IF (p_provincia IS NULL OR p_provincia = '') THEN
        RAISE e_null_prov;
    ELSIF (p_region IS NULL OR p_region ='') THEN
        RAISE e_null_region;
    ELSIF (p_pais IS NULL OR p_pais ='') THEN
        RAISE e_null_pais;
    END IF;

    -- Asignación del valor OK a la variable de salida
    -- Selección de la operación a realizar 
    CASE 
    WHEN p_tipo_oper = 'INSERT' THEN
        -- Inserción en la tabla.
        INSERT INTO Demarcacion VALUES (p_provincia, p_region, p_pais);
        RSP := 'OK';
        -- Insertar el resultado en la tabla test_log
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    WHEN p_tipo_oper = 'UPDATE' THEN
        RSP := 'OK';
        UPDATE Demarcacion SET REGION = p_region WHERE PROVINCIA = p_provincia;
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    WHEN p_tipo_oper = 'DELETE' THEN
        RSP := 'OK';
        DELETE FROM Demarcacion WHERE PROVINCIA = p_provincia;
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    END CASE;

    -- Tratamiento de excepciones
    EXCEPTION
        WHEN e_null_prov THEN
            RSP := 'ERROR: PROVINCIA nula';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_region THEN
            RSP := 'ERROR: REGION nula';    
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
             VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_pais THEN
            RSP := 'ERROR: PAIS nulo';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_no_pk_ref THEN
            RSP := 'Error: No existe la clave principal referenciada';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_fk THEN
            RSP := 'Error: Existe una clave foranea que referencia a Demarcacion';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN DUP_VAL_ON_INDEX THEN
            RSP := 'ERROR: Ya existe una Demarcacion con esta PK';  
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN NO_DATA_FOUND THEN
            RSP := 'ERROR: No existen datos con los parametros dados';  
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN OTHERS THEN
            RSP := 'ERROR: ' || SUBSTR(SQLERRM(-SQLCODE),1,100); 
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);

END p_test_demarcacion;

-- Procedimiento para el test de la tabla Recibe

create or replace PROCEDURE p_test_recibe (
    p_fec_conv IN RECIBE.fechaConvBOE%TYPE,
    p_ambito IN RECIBE.AMBITO%TYPE,
    p_provincia IN RECIBE.PROVINCIA%TYPE, 
    p_num_esc IN RECIBE.NUMESCANOS%TYPE,
    p_tabla_mod IN VARCHAR2,
    p_tipo_oper IN VARCHAR2,
    RSP OUT VARCHAR2)
AS
    v_proced TEST_LOG.PROCEDIMIENTO%TYPE := 'p_test_recibe';

    e_null_fec_conv EXCEPTION;
    e_null_ambito EXCEPTION;
    e_enum_ambito EXCEPTION;
    e_null_prov EXCEPTION;
    e_null_num_esc EXCEPTION;
    e_num_esc_neg EXCEPTION;
    e_no_pk_ref EXCEPTION;
    pragma exception_init(e_no_pk_ref, -2291);
    e_fk EXCEPTION;
    pragma exception_init(e_fk, -2292);

BEGIN
    -- Inicio del procedimiento

    -- Comprobación de parámetros
    IF (p_fec_conv IS NULL OR p_fec_conv = '') THEN
        RAISE e_null_fec_conv;
    ELSIF (p_ambito IS NULL OR p_ambito ='') THEN
        RAISE e_null_ambito;
    ELSIF (p_ambito NOT IN ('Generales', 'Municipales', 'Parlamento Europeo', 'Autonomicas')) THEN
        RAISE e_enum_ambito;
    ELSIF (p_provincia IS NULL OR p_provincia = '') THEN
        RAISE e_null_prov;
    ELSIF (p_num_esc IS NULL OR p_num_esc ='') THEN
        RAISE e_null_num_esc;
    ELSIF (p_num_esc < 0) THEN
        RAISE e_num_esc_neg;
    END IF;

    -- Asignación del valor OK a la variable de salida
    -- Selección de la operación a realizar 
    CASE 
    WHEN p_tipo_oper = 'INSERT' THEN
        -- Inserción en la tabla.
        INSERT INTO RECIBE VALUES (p_fec_conv, p_ambito, p_provincia, p_num_esc);
        RSP := 'OK';
        -- Insertar el resultado en la tabla test_log
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    WHEN p_tipo_oper = 'UPDATE' THEN
        RSP := 'OK';
        UPDATE RECIBE SET AMBITO = p_ambito WHERE FECHACONVBOE = p_fec_conv;
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    WHEN p_tipo_oper = 'DELETE' THEN
        RSP := 'OK';
        DELETE FROM RECIBE WHERE FECHACONVBOE = p_fec_conv;
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    END CASE;

    -- Tratamiento de excepciones
    EXCEPTION
        WHEN e_null_fec_conv THEN
            RSP := 'ERROR: Fecha convocatoria nula';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_ambito THEN
            RSP := 'ERROR: Ambito de elecciones nulo';  
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_enum_ambito THEN
            RSP := 'ERROR: Ambito de elecciones no válido';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_prov THEN
            RSP := 'ERROR: PROVINCIA nula';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_num_esc THEN
            RSP := 'ERROR: Numero de escaños nulo'; 
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
             VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_num_esc_neg THEN
            RSP := 'ERROR: Numero de escaños negativo';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_no_pk_ref THEN
            RSP := 'Error: No existe la clave principal referenciada';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_fk THEN
            RSP := 'Error: Existe una clave foranea que referencia a RECIBE';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN DUP_VAL_ON_INDEX THEN
            RSP := 'ERROR: Ya existe una RECIBE con esta PK';   
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN NO_DATA_FOUND THEN
            RSP := 'ERROR: No existen datos con los parametros dados';  
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN OTHERS THEN
            RSP := 'ERROR: ' || SUBSTR(SQLERRM(-SQLCODE),1,100); 
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);

END p_test_recibe;


-- Procedimiento para el test de la tabla CentroVotacion

create or replace PROCEDURE p_test_centro_vot (
    p_municipio IN CENTROVOTACION.MUNICIPIO%TYPE,
    p_provincia IN CENTROVOTACION.PROVINCIA%TYPE, 
    p_tabla_mod IN VARCHAR2,
    p_tipo_oper IN VARCHAR2,
    RSP OUT VARCHAR2)
AS
    v_proced TEST_LOG.PROCEDIMIENTO%TYPE := 'p_test_centro_vot';

    e_null_prov EXCEPTION;
    e_null_municipio EXCEPTION;
    e_no_pk_ref EXCEPTION;
    pragma exception_init(e_no_pk_ref, -2291);
    e_fk EXCEPTION;
    pragma exception_init(e_fk, -2292);

BEGIN
    -- Inicio del procedimiento

    -- Comprobación de parámetros
    IF (p_provincia IS NULL OR p_provincia = '') THEN
        RAISE e_null_prov;
    ELSIF (p_municipio IS NULL OR p_municipio ='') THEN
        RAISE e_null_municipio;
    END IF;

    -- Asignación del valor OK a la variable de salida
    -- Selección de la operación a realizar 
    CASE 
    WHEN p_tipo_oper = 'INSERT' THEN
        -- Inserción en la tabla.
        INSERT INTO CENTROVOTACION VALUES (p_municipio, p_provincia);
        RSP := 'OK';
        -- Insertar el resultado en la tabla test_log
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    WHEN p_tipo_oper = 'UPDATE' THEN
        RSP := 'OK';
        UPDATE CENTROVOTACION SET MUNICIPIO = p_municipio WHERE PROVINCIA = p_provincia;
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    WHEN p_tipo_oper = 'DELETE' THEN
        RSP := 'OK';
        DELETE FROM CENTROVOTACION WHERE MUNICIPIO = p_municipio;
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    END CASE;

    -- Tratamiento de excepciones
    EXCEPTION
        WHEN e_null_prov THEN
            RSP := 'ERROR: PROVINCIA nula';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_municipio THEN
            RSP := 'ERROR: MUNICIPIO nula';    
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
             VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_no_pk_ref THEN
            RSP := 'Error: No existe la clave principal referenciada';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_fk THEN
            RSP := 'Error: Existe una clave foranea que referencia a CENTROVOTACION';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN DUP_VAL_ON_INDEX THEN
            RSP := 'ERROR: Ya existe una CENTROVOTACION con esta PK';  
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN NO_DATA_FOUND THEN
            RSP := 'ERROR: No existen datos con los parametros dados';  
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN OTHERS THEN
            RSP := 'ERROR: ' || SUBSTR(SQLERRM(-SQLCODE),1,100); 
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);

END p_test_centro_vot;

-- Procedimiento para el test de la tabla ComunicacionPersonal

create or replace PROCEDURE p_test_com_per (
    p_fecha_com IN COMUNICACIONPERSONAL.FECHACOMPERSONAL%TYPE,
    p_tabla_mod IN VARCHAR2,
    p_tipo_oper IN VARCHAR2,
    RSP OUT VARCHAR2)
AS
    v_proced TEST_LOG.PROCEDIMIENTO%TYPE := 'p_test_com_per';

    e_null_fecha_com EXCEPTION;
    e_no_pk_ref EXCEPTION;
    pragma exception_init(e_no_pk_ref, -2291);
    e_fk EXCEPTION;
    pragma exception_init(e_fk, -2292);

BEGIN
    -- Inicio del procedimiento

    -- Comprobación de parámetros

    IF (p_fecha_com IS NULL OR p_fecha_com ='') THEN
        RAISE e_null_fecha_com;
    END IF;

    -- Asignación del valor OK a la variable de salida
    -- Selección de la operación a realizar 
    CASE 
    WHEN p_tipo_oper = 'INSERT' THEN
        -- Inserción en la tabla.
        INSERT INTO COMUNICACIONPERSONAL VALUES (p_fecha_com);
        RSP := 'OK';
        -- Insertar el resultado en la tabla test_log
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    WHEN p_tipo_oper = 'UPDATE' THEN
        RSP := 'OK';
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    WHEN p_tipo_oper = 'DELETE' THEN
        RSP := 'OK';
        DELETE FROM COMUNICACIONPERSONAL WHERE FECHACOMPERSONAL = p_fecha_com;
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    END CASE;

    -- Tratamiento de excepciones
    EXCEPTION
        WHEN e_null_fecha_com THEN
            RSP := 'ERROR: Fecha de comunicación personal nula';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_no_pk_ref THEN
            RSP := 'Error: No existe la clave principal referenciada';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_fk THEN
            RSP := 'Error: Existe una clave foranea que referencia a COMUNICACIONPERSONAL';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN DUP_VAL_ON_INDEX THEN
            RSP := 'ERROR: Ya existe una COMUNICACIONPERSONAL con esta PK';  
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN NO_DATA_FOUND THEN
            RSP := 'ERROR: No existen datos con los parametros dados';  
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN OTHERS THEN
            RSP := 'ERROR: ' || SUBSTR(SQLERRM(-SQLCODE),1,100); 
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);

END p_test_com_per;

-- Procedimiento para el test de la tabla CensoElectoral

create or replace PROCEDURE p_test_censo (
    p_fecha_censo IN CENSOELECTORAL.FECHACIERRE%TYPE,
    p_tabla_mod IN VARCHAR2,
    p_tipo_oper IN VARCHAR2,
    RSP OUT VARCHAR2)
AS
    v_proced TEST_LOG.PROCEDIMIENTO%TYPE := 'p_test_censo';

    e_null_fecha_cen EXCEPTION;
    e_fk EXCEPTION;
    pragma exception_init(e_fk, -2292);

BEGIN
    -- Inicio del procedimiento

    -- Comprobación de parámetros

    IF (p_fecha_censo IS NULL OR p_fecha_censo ='') THEN
        RAISE e_null_fecha_cen;
    END IF;

    -- Asignación del valor OK a la variable de salida
    -- Selección de la operación a realizar 
    CASE 
    WHEN p_tipo_oper = 'INSERT' THEN
        -- Inserción en la tabla.
        INSERT INTO CENSOELECTORAL VALUES (p_fecha_censo);
        RSP := 'OK';
        -- Insertar el resultado en la tabla test_log
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    WHEN p_tipo_oper = 'UPDATE' THEN
        RSP := 'OK';
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    WHEN p_tipo_oper = 'DELETE' THEN
        RSP := 'OK';
        DELETE FROM CENSOELECTORAL WHERE FECHACIERRE = p_fecha_censo;
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    END CASE;

    -- Tratamiento de excepciones
    EXCEPTION
        WHEN e_null_fecha_cen THEN
            RSP := 'ERROR: Fecha del censo nula';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_fk THEN
            RSP := 'Error: Existe una clave foranea que referencia a CENSOELECTORAL';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN DUP_VAL_ON_INDEX THEN
            RSP := 'ERROR: Ya existe una CENSOELECTORAL con esta PK';  
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN NO_DATA_FOUND THEN
            RSP := 'ERROR: No existen datos con los parametros dados';  
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN OTHERS THEN
            RSP := 'ERROR: ' || SUBSTR(SQLERRM(-SQLCODE),1,100); 
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);

END p_test_censo;

-- Procedimiento para el test de la tabla MesaElectoral

create or replace PROCEDURE p_test_mesa_electoral (
    p_mesaId IN MESAELECTORAL.MESAID%TYPE,
    p_fecha_proc IN MESAELECTORAL.FECHAPROCESO%TYPE,
    p_ambito IN MESAELECTORAL.AMBITO%TYPE,
    p_cod_pos IN MESAELECTORAL.CODIGOPOSTAL%TYPE,
    p_calle IN MESAELECTORAL.CALLE%TYPE,
    p_num IN MESAELECTORAL.NUMERO%TYPE,
    p_municipio IN MESAELECTORAL.MUNICIPIO%TYPE,
    p_demarc IN MESAELECTORAL.DEMARCACION%TYPE,
    p_num_votant IN MESAELECTORAL.NUMEROVOTANTES%TYPE,
    p_num_votos IN MESAELECTORAL.NUMEROVOTOS%TYPE,
    p_tabla_mod IN VARCHAR2,
    p_tipo_oper IN VARCHAR2,
    RSP OUT VARCHAR2)
AS
    v_proced TEST_LOG.PROCEDIMIENTO%TYPE := 'p_test_mesa_electoral';

    e_null_mesaId EXCEPTION;
    e_null_fecha_proc EXCEPTION;
    e_null_ambito EXCEPTION;
    e_enum_ambito EXCEPTION;
    e_null_cp EXCEPTION;
    e_null_calle EXCEPTION;
    e_null_num EXCEPTION;
    e_null_munic EXCEPTION;
    e_null_demarc EXCEPTION;
    e_num_votantes EXCEPTION;
    e_num_votos EXCEPTION;
    e_votos_mayor EXCEPTION;
    e_no_pk_ref EXCEPTION;
    pragma exception_init(e_no_pk_ref, -2291);
    e_fk EXCEPTION;
    pragma exception_init(e_fk, -2292);

BEGIN
    -- Inicio del procedimiento

    -- Comprobación de parámetros
    IF  (p_mesaId IS NULL OR p_mesaId ='') THEN
        RAISE e_null_mesaId;
    ELSIF (p_fecha_proc IS NULL OR p_fecha_proc ='') THEN
        RAISE e_null_fecha_proc;
    ELSIF (p_ambito IS NULL OR p_ambito ='') THEN
        RAISE e_null_ambito;
    ELSIF (p_ambito NOT IN ('Generales', 'Municipales', 'Parlamento Europeo', 'Autonomicas')) THEN
        RAISE e_enum_ambito;
    ELSIF  (p_cod_pos IS NULL OR p_cod_pos ='') THEN
        RAISE e_null_cp;
    ELSIF  (p_calle IS NULL OR p_calle ='') THEN
        RAISE e_null_calle;
    ELSIF  (p_num IS NULL OR p_num ='') THEN
        RAISE e_null_num;
    ELSIF  (p_municipio IS NULL OR p_municipio ='') THEN
        RAISE e_null_munic;
    ELSIF  (p_demarc IS NULL OR p_demarc ='') THEN
        RAISE e_null_demarc;
    ELSIF  (p_num_votant < 0) THEN
        RAISE e_num_votantes;
    ELSIF  (p_num_votos < 0) THEN
        RAISE e_num_votos;
    ELSIF  (p_num_votos > p_num_votant) THEN
        RAISE e_votos_mayor;
    END IF;

    -- Asignación del valor OK a la variable de salida
    -- Selección de la operación a realizar 
    CASE 
    WHEN p_tipo_oper = 'INSERT' THEN
        -- Inserción en la tabla.
        INSERT INTO MESAELECTORAL (MESAID, FECHAPROCESO, AMBITO, CODIGOPOSTAL, CALLE, NUMERO, 
            MUNICIPIO, DEMARCACION, NUMEROVOTANTES, NUMEROVOTOS) 
        VALUES (p_mesaId, p_fecha_proc, p_ambito, p_cod_pos, p_calle, p_num, 
                p_municipio, p_demarc, p_num_votant, p_num_votos);
        RSP := 'OK';
        -- Insertar el resultado en la tabla test_log
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    WHEN p_tipo_oper = 'UPDATE' THEN
        RSP := 'OK';
        UPDATE MESAELECTORAL SET AMBITO = 'Municipales' WHERE MESAID = p_mesaId;
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    WHEN p_tipo_oper = 'DELETE' THEN
        RSP := 'OK';
        DELETE FROM MESAELECTORAL WHERE MESAID = p_mesaId;
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    END CASE;

    -- Tratamiento de excepciones
    EXCEPTION
        WHEN e_null_mesaId THEN
            RSP := 'ERROR: Identificador de mesa nulo';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_fecha_proc THEN
            RSP := 'ERROR: Fecha del proceso electoral nula';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_ambito THEN
            RSP := 'ERROR: Ambito de elecciones nulo';  
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
             VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_enum_ambito THEN
            RSP := 'ERROR: Ambito de elecciones no válido';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_cp THEN
            RSP := 'ERROR: Codigo postal nulo';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
         WHEN e_null_calle THEN
            RSP := 'ERROR: Nombre de calle nulo';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_num THEN
            RSP := 'ERROR: Número de la calle nulo';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_munic THEN
            RSP := 'ERROR: Municipio nulo';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_demarc THEN
            RSP := 'ERROR: Demarcacion nula';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_num_votantes THEN
            RSP := 'ERROR: El numero de votantes debe ser cero o mayor';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_num_votos THEN
            RSP := 'ERROR: El numero de votos debe ser cero o mayor';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_votos_mayor THEN
            RSP := 'ERROR: Recuento de votos. Hay follón!!!';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_no_pk_ref THEN
            RSP := 'Error: No existe la clave principal referenciada';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_fk THEN
            RSP := 'Error: Existe una clave foranea que referencia a MESAELECTORAL';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN DUP_VAL_ON_INDEX THEN
            RSP := 'ERROR: Ya existe una MESAELECTORAL con esta PK';  
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN NO_DATA_FOUND THEN
            RSP := 'ERROR: No existen datos con los parametros dados';  
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN OTHERS THEN
            RSP := 'ERROR: ' || SUBSTR(SQLERRM(-SQLCODE),1,100); 
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);

END p_test_mesa_electoral;

-- Procedimiento para el test de la tabla Persona

create or replace PROCEDURE p_test_persona (
    p_dni IN PERSONA.DNI%TYPE,
    p_fec_cierre IN PERSONA.FECHACIERRE%TYPE,
    p_nombre IN PERSONA.NOMBRE%TYPE,
    p_apellidos IN PERSONA.APELLIDOS%TYPE,
    p_sexo IN PERSONA.SEXO%TYPE,
    p_nac IN PERSONA.NACIONALIDAD%TYPE,
    p_fecha_nac IN PERSONA.FECHANACIMIENTO%TYPE,
    p_lugar_nac IN PERSONA.LUGARNACIMIENTO%TYPE,
    p_estudios IN PERSONA.ESTUDIOS%TYPE,
    p_tfno IN PERSONA.TELEFONO%TYPE,
    p_proces IN PERSONA.PROCESADO%TYPE,
    p_direc IN PERSONA.DIRECCION%TYPE,
    p_tabla_mod IN VARCHAR2,
    p_tipo_oper IN VARCHAR2,
    RSP OUT VARCHAR2)
AS
    v_proced TEST_LOG.PROCEDIMIENTO%TYPE := 'p_test_persona';

    e_null_dni EXCEPTION;
    e_null_fecha_cierre EXCEPTION;
    e_null_nombre EXCEPTION;
    e_null_apellid EXCEPTION;
    e_null_sexo EXCEPTION;
    e_null_nacionalidad EXCEPTION;
    e_null_fec_nac EXCEPTION;
    e_menor_fecha_nac EXCEPTION;
    e_null_estud EXCEPTION;
    e_null_proces EXCEPTION;
    e_enum_estud EXCEPTION;
    e_enum_proces EXCEPTION;
    e_enum_sexo EXCEPTION;
    e_no_pk_ref EXCEPTION;
    pragma exception_init(e_no_pk_ref, -2291);
    e_fk EXCEPTION;
    pragma exception_init(e_fk, -2292);

BEGIN
    -- Inicio del procedimiento

    -- Comprobación de parámetros
    IF  (p_dni IS NULL OR p_dni ='') THEN
        RAISE e_null_dni;
    ELSIF (p_fec_cierre IS NULL OR p_fec_cierre ='') THEN
        RAISE e_null_fecha_cierre;
    ELSIF (p_nombre IS NULL OR p_nombre ='') THEN
        RAISE e_null_nombre;
    ELSIF (p_apellidos IS NULL OR p_apellidos ='') THEN
        RAISE e_null_apellid;
    ELSIF  (p_sexo IS NULL OR p_sexo ='') THEN
        RAISE e_null_sexo;
    ELSIF  (p_nac IS NULL OR p_nac ='') THEN
        RAISE e_null_nacionalidad;
    ELSIF  (p_fecha_nac IS NULL OR p_fecha_nac ='') THEN
        RAISE e_null_fec_nac;
    ELSIF  (p_fecha_nac > SYSDATE) THEN
        RAISE e_menor_fecha_nac;
    ELSIF  (p_estudios IS NULL OR p_estudios ='') THEN
        RAISE e_null_estud;
    ELSIF  (p_proces IS NULL OR p_proces ='') THEN
        RAISE e_null_proces;
    ELSIF (p_estudios NOT IN ('ESO', 'Bachillerato', 'FP', 'Universitarios')) THEN
         RAISE e_enum_estud;
    ELSIF (p_proces NOT IN ('SI','NO')) THEN
        RAISE e_enum_proces;
    ELSIF (p_sexo NOT IN ('Hombre','Mujer')) THEN
        RAISE e_enum_sexo;

    END IF;

    -- Asignación del valor OK a la variable de salida
    -- Selección de la operación a realizar 
    CASE 
    WHEN p_tipo_oper = 'INSERT' THEN
        -- Inserción en la tabla.
        INSERT INTO PERSONA
        VALUES (p_dni, p_fec_cierre, p_nombre, p_apellidos, p_sexo, p_nac, 
                p_fecha_nac, p_lugar_nac, p_estudios, p_tfno, p_proces, p_direc);
        RSP := 'OK';
        -- Insertar el resultado en la tabla test_log
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    WHEN p_tipo_oper = 'UPDATE' THEN
        RSP := 'OK';
        UPDATE PERSONA SET NOMBRE = 'Manué' WHERE DNI = p_dni;
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    WHEN p_tipo_oper = 'DELETE' THEN
        RSP := 'OK';
        DELETE FROM PERSONA WHERE DNI = p_dni AND FECHACIERRE = p_fec_cierre;
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    END CASE;

    -- Tratamiento de excepciones
    EXCEPTION
        WHEN e_null_dni THEN
            RSP := 'ERROR: DNI nulo';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_fecha_cierre THEN
            RSP := 'ERROR: Fecha de cierre del censo nula';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_nombre THEN
            RSP := 'ERROR: Nombre de la persona nulo';  
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
             VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_apellid THEN
            RSP := 'ERROR: Apellidos de la persona nulos';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_sexo THEN
            RSP := 'ERROR: Sexo nulo';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
         WHEN e_null_nacionalidad THEN
            RSP := 'ERROR: Nacionalidad nula';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_fec_nac THEN
            RSP := 'ERROR: Fecha de nacimiento nula';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_menor_fecha_nac THEN
            RSP := 'ERROR: Fecha de nacimiento posterior a la actual';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_estud THEN
            RSP := 'ERROR: Campo estudos nulo';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_proces THEN
            RSP := 'ERROR: Campo procesado nulo';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_enum_estud THEN
            RSP := 'ERROR: Nivel de estudios no válido';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_enum_proces THEN
            RSP := 'ERROR: Estado procesal no válido';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_enum_sexo THEN
            RSP := 'ERROR: Valor sexo no válido';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_no_pk_ref THEN
            RSP := 'Error: No existe la clave principal referenciada';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_fk THEN
            RSP := 'Error: Existe una clave foranea que referencia a PERSONA';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN DUP_VAL_ON_INDEX THEN
            RSP := 'ERROR: Ya existe una PERSONA con esta PK';  
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN NO_DATA_FOUND THEN
            RSP := 'ERROR: No existen datos con los parametros dados';  
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN OTHERS THEN
            RSP := 'ERROR: ' || SUBSTR(SQLERRM(-SQLCODE),1,100); 
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);

END p_test_persona;

-- Procedimiento para el test de la tabla Candidato

create or replace PROCEDURE p_test_candidato (
    p_dni IN CANDIDATO.DNI%TYPE,
    p_fec_cierre IN CANDIDATO.FECHACIERRE%TYPE,
    p_es_suplente IN CANDIDATO.ESSUPLENTE%TYPE,
    p_tabla_mod IN VARCHAR2,
    p_tipo_oper IN VARCHAR2,
    RSP OUT VARCHAR2)
AS
    v_proced TEST_LOG.PROCEDIMIENTO%TYPE := 'p_test_candidato';

    e_null_dni EXCEPTION;
    e_null_fecha_cierre EXCEPTION;
    e_null_esSuplente EXCEPTION;
    e_enum_esSuplente EXCEPTION;
    e_no_pk_ref EXCEPTION;
    pragma exception_init(e_no_pk_ref, -2291);
    e_fk EXCEPTION;
    pragma exception_init(e_fk, -2292);

BEGIN
    -- Inicio del procedimiento

    -- Comprobación de parámetros
    IF  (p_dni IS NULL OR p_dni ='') THEN
        RAISE e_null_dni;
    ELSIF (p_fec_cierre IS NULL OR p_fec_cierre ='') THEN
        RAISE e_null_fecha_cierre;
    ELSIF (p_es_suplente IS NULL OR p_es_suplente ='') THEN
        RAISE e_null_esSuplente;        
    ELSIF (p_es_suplente NOT IN (0,1)) THEN
        RAISE e_enum_esSuplente;
    END IF;

    -- Asignación del valor OK a la variable de salida
    -- Selección de la operación a realizar 
    CASE 
    WHEN p_tipo_oper = 'INSERT' THEN
        -- Inserción en la tabla.
        INSERT INTO CANDIDATO
        VALUES (p_dni, p_fec_cierre, p_es_suplente);
        RSP := 'OK';
        -- Insertar el resultado en la tabla test_log
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    WHEN p_tipo_oper = 'UPDATE' THEN
        RSP := 'OK';
        UPDATE CANDIDATO SET ESSUPLENTE = 1 WHERE DNI = p_dni AND FECHACIERRE = p_fec_cierre;
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    WHEN p_tipo_oper = 'DELETE' THEN
        RSP := 'OK';
        DELETE FROM CANDIDATO WHERE DNI = p_dni AND FECHACIERRE = p_fec_cierre;
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    END CASE;

    -- Tratamiento de excepciones
    EXCEPTION
        WHEN e_null_dni THEN
            RSP := 'ERROR: DNI nulo';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_fecha_cierre THEN
            RSP := 'ERROR: Fecha de cierre del censo nula';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_esSuplente THEN
            RSP := 'ERROR: Es necesario especificar si el candidato es suplente o no';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_enum_esSuplente THEN
            RSP := 'ERROR: La columna esSuplente debe tener valores 0 ó 1';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_no_pk_ref THEN
            RSP := 'Error: No existe la clave principal referenciada';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_fk THEN
            RSP := 'Error: Existe una clave foranea que referencia a CANDIDATO';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN DUP_VAL_ON_INDEX THEN
            RSP := 'ERROR: Ya existe un CANDIDATO con esta PK';  
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN NO_DATA_FOUND THEN
            RSP := 'ERROR: No existen datos con los parametros dados';  
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN OTHERS THEN
            RSP := 'ERROR: ' || SUBSTR(SQLERRM(-SQLCODE),1,100); 
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);

END p_test_candidato;

-- Procedimiento para el test de la tabla Presidente

create or replace PROCEDURE p_test_presidente (
    p_dni IN PRESIDENTE.DNI%TYPE,
    p_fec_cierre IN PRESIDENTE.FECHACIERRE%TYPE,
    p_num_aleg IN PRESIDENTE.NUMALEGACIONES%TYPE,
    p_num_partic IN PRESIDENTE.NUMPARTICIPACIONES%TYPE,
    p_tabla_mod IN VARCHAR2,
    p_tipo_oper IN VARCHAR2,
    RSP OUT VARCHAR2)
AS
    v_proced TEST_LOG.PROCEDIMIENTO%TYPE := 'p_test_presidente';
    v_dob DATE;
    v_years INTEGER;

    e_null_dni EXCEPTION;
    e_null_fecha_cierre EXCEPTION;
    e_null_aleg EXCEPTION;
    e_null_part EXCEPTION;
    e_num_aleg_neg EXCEPTION;
    e_num_part_neg EXCEPTION;
    e_mayor_70 EXCEPTION;
    e_no_pk_ref EXCEPTION;
    pragma exception_init(e_no_pk_ref, -2291);
    e_fk EXCEPTION;
    pragma exception_init(e_fk, -2292);

BEGIN
    -- Inicio del procedimiento
    SELECT FECHANACIMIENTO INTO v_dob FROM PERSONA WHERE DNI = p_dni AND FECHACIERRE = p_fec_cierre;
    SELECT FLOOR(months_between(p_fec_cierre, v_dob) /12) INTO v_years FROM DUAL;
    -- Comprobación de parámetros
    IF  (p_dni IS NULL OR p_dni ='') THEN
        RAISE e_null_dni;
    ELSIF (p_fec_cierre IS NULL OR p_fec_cierre ='') THEN
        RAISE e_null_fecha_cierre;
    ELSIF (p_num_aleg IS NULL OR p_num_aleg ='') THEN
        RAISE e_null_aleg;
    ELSIF (p_num_partic IS NULL OR p_num_partic ='') THEN
        RAISE e_null_part;
    ELSIF  (p_num_aleg < 0) THEN
        RAISE e_num_aleg_neg;
    ELSIF  (p_num_partic < 0) THEN
        RAISE e_num_part_neg;
    ELSIF (v_years >=70) THEN
        RAISE e_mayor_70;
    END IF;

    -- Asignación del valor OK a la variable de salida
    -- Selección de la operación a realizar 
    CASE 
    WHEN p_tipo_oper = 'INSERT' THEN
        -- Inserción en la tabla.
        INSERT INTO PRESIDENTE
        VALUES (p_dni, p_fec_cierre, p_num_aleg, p_num_partic);
        RSP := 'OK';
        -- Insertar el resultado en la tabla test_log
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    WHEN p_tipo_oper = 'UPDATE' THEN
        RSP := 'OK';
        UPDATE PRESIDENTE SET NUMALEGACIONES = 1 WHERE DNI = p_dni;
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    WHEN p_tipo_oper = 'DELETE' THEN
        RSP := 'OK';
        DELETE FROM PRESIDENTE WHERE DNI = p_dni;
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    END CASE;

    -- Tratamiento de excepciones
    EXCEPTION
        WHEN e_null_dni THEN
            RSP := 'ERROR: DNI nulo';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_fecha_cierre THEN
            RSP := 'ERROR: Fecha de cierre del censo nula';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_aleg THEN
            RSP := 'ERROR: Número de alegaciones nulo';  
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
             VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_part THEN
            RSP := 'ERROR: Numero de participaciones nulo';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_num_aleg_neg THEN
            RSP := 'ERROR: Valor negativo del número de alegaciones';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
         WHEN e_num_part_neg THEN
            RSP := 'ERROR: Valor negativo del número de participaciones';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_mayor_70 THEN
            RSP := 'ERROR: El presidente debe tener menos de 70 años';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_no_pk_ref THEN
            RSP := 'Error: No existe la clave principal referenciada';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_fk THEN
            RSP := 'Error: Existe una clave foranea que referencia a PRESIDENTE';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN DUP_VAL_ON_INDEX THEN
            RSP := 'ERROR: Ya existe una PRESIDENTE con esta PK';  
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN NO_DATA_FOUND THEN
            RSP := 'ERROR: No existen datos con los parametros dados';  
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN OTHERS THEN
            RSP := 'ERROR: ' || SUBSTR(SQLERRM(-SQLCODE),1,100); 
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);

END p_test_presidente;

-- Procedimiento para el test de la tabla Compuesta

create or replace PROCEDURE p_test_compuesta (
    p_mesaId IN COMPUESTA.MESAID%TYPE,
    p_fec_cierre IN COMPUESTA.FECHACIERRE%TYPE,
    p_dni IN COMPUESTA.DNI%TYPE,
    p_fec_ini_sort IN COMPUESTA.FECHAINISORTEO%TYPE,
    p_fec_sort IN COMPUESTA.FECHASORTEO%TYPE,
    p_fec_ini_notif IN COMPUESTA.FECHAININOTIF%TYPE,
    p_fec_notif IN COMPUESTA.FECHANOTIF%TYPE,
    p_fec_ini_aleg IN COMPUESTA.FECHAINIALEGAC%TYPE,
    p_fec_aleg IN COMPUESTA.FECHAALEGACION%TYPE,
    p_tabla_mod IN VARCHAR2,
    p_tipo_oper IN VARCHAR2,
    RSP OUT VARCHAR2)
AS
    v_proced TEST_LOG.PROCEDIMIENTO%TYPE := 'p_test_compuesta';
    v_num_miembros INTEGER;
    
    e_null_mesaId EXCEPTION;
    e_null_fecha_cierre EXCEPTION;
    e_null_dni EXCEPTION;
    e_null_fec_ini_sort EXCEPTION;
    e_plazo_sort EXCEPTION;
    e_plazo_notif EXCEPTION;
    e_plazo_aleg EXCEPTION;
    e_num_miembros EXCEPTION;
    e_no_pk_ref EXCEPTION;
    pragma exception_init(e_no_pk_ref, -2291);
    e_fk EXCEPTION;
    pragma exception_init(e_fk, -2292);

BEGIN
    -- Inicio del procedimiento
    SELECT COUNT(*) INTO v_num_miembros FROM COMPUESTA WHERE mesaId = p_mesaId;
    -- Comprobación de parámetros
    IF (p_mesaId IS NULL OR p_mesaId ='') THEN
        RAISE e_null_mesaId;
    ELSIF (p_dni IS NULL OR p_dni ='') THEN
        RAISE e_null_dni;
    ELSIF (p_fec_cierre IS NULL OR p_fec_cierre ='') THEN
        RAISE e_null_fecha_cierre;
    ELSIF (p_fec_ini_sort IS NULL OR p_fec_ini_sort ='') THEN
        RAISE e_null_fec_ini_sort;
    ELSIF (p_fec_sort NOT BETWEEN p_fec_ini_sort AND p_fec_ini_sort + 4) THEN
        RAISE e_plazo_sort;
    ELSIF (p_fec_notif NOT BETWEEN p_fec_ini_notif AND p_fec_ini_notif + 3) THEN
        RAISE e_plazo_notif;
    ELSIF (p_fec_aleg NOT BETWEEN p_fec_ini_aleg AND p_fec_ini_aleg + 7) THEN
        RAISE e_plazo_aleg;
    ELSIF (v_num_miembros >= 6 AND p_tipo_oper = 'INSERT') THEN
        RAISE e_num_miembros;
    END IF;

    -- Asignación del valor OK a la variable de salida
    -- Selección de la operación a realizar 
    CASE 
    WHEN p_tipo_oper = 'INSERT' THEN
        -- Inserción en la tabla.
        INSERT INTO COMPUESTA
        VALUES (p_mesaId, p_fec_cierre, p_dni, p_fec_ini_sort, p_fec_sort, 
                p_fec_ini_notif, p_fec_notif, p_fec_ini_aleg, p_fec_aleg);
        RSP := 'OK';
        -- Insertar el resultado en la tabla test_log
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    WHEN p_tipo_oper = 'UPDATE' THEN
        RSP := 'OK';
        UPDATE COMPUESTA SET DNI = 'upDNI' WHERE MESAID = p_mesaId;
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    WHEN p_tipo_oper = 'DELETE' THEN
        RSP := 'OK';
        DELETE FROM COMPUESTA WHERE MESAID = p_mesaId;
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    END CASE;

    -- Tratamiento de excepciones
    EXCEPTION
        WHEN e_null_mesaId THEN
            RSP := 'ERROR: Identificador de mesa nulo';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_dni THEN
            RSP := 'ERROR: DNI nulo';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_fecha_cierre THEN
            RSP := 'ERROR: Fecha de cierre del censo nula';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_fec_ini_sort THEN
            RSP := 'ERROR: Fecha de inicio de sorteo nula';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_plazo_sort THEN
            RSP := 'ERROR: Fecha de sorteo fuera de plazo';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_plazo_notif THEN
            RSP := 'ERROR: Fecha de notificación fuera de plazo';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_plazo_aleg THEN
            RSP := 'ERROR: Fecha de alegación fuera de plazo';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_num_miembros THEN
            RSP := 'ERROR: Una mesa no puede tener mas de 6 miembros';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_no_pk_ref THEN
            RSP := 'Error: No existe la clave principal referenciada';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_fk THEN
            RSP := 'Error: Existe una clave foranea que referencia a COMPUESTA';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN DUP_VAL_ON_INDEX THEN
            RSP := 'ERROR: Ya existe una COMPUESTA con esta PK';  
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN NO_DATA_FOUND THEN
            RSP := 'ERROR: No existen datos con los parametros dados';  
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN OTHERS THEN
            RSP := 'ERROR: ' || SUBSTR(SQLERRM(-SQLCODE),1,100); 
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);

END p_test_compuesta;

-- Procedimiento para el test de la tabla ReglasMiembroMesa

create or replace PROCEDURE p_test_reglas_miembro_mesa (
    p_mesaId IN REGLASMIEMBROMESA.MESAID%TYPE, 
    p_num_regla IN REGLASMIEMBROMESA.NREGLA%TYPE,
    p_regla IN REGLASMIEMBROMESA.REGLA%TYPE,
    p_tabla_mod IN VARCHAR2,
    p_tipo_oper IN VARCHAR2,
    RSP OUT VARCHAR2)
AS
    v_proced TEST_LOG.PROCEDIMIENTO%TYPE := 'p_test_reglas_miembro_mesa';

    e_null_mesaId EXCEPTION;
    e_null_n_regla EXCEPTION;
    e_no_pk_ref EXCEPTION;
    pragma exception_init(e_no_pk_ref, -2291);
    e_fk EXCEPTION;
    pragma exception_init(e_fk, -2292);

BEGIN
    -- Inicio del procedimiento

    -- Comprobación de parámetros
    IF (p_mesaId IS NULL OR p_mesaId = '') THEN
        RAISE e_null_mesaId;
    ELSIF (p_num_regla IS NULL OR p_num_regla ='') THEN
        RAISE e_null_n_regla;
    END IF;

    -- Asignación del valor OK a la variable de salida
    -- Selección de la operación a realizar 
    CASE 
    WHEN p_tipo_oper = 'INSERT' THEN
        -- Inserción en la tabla.
        INSERT INTO REGLASMIEMBROMESA VALUES (p_mesaId, p_num_regla, p_regla);
        RSP := 'OK';
        -- Insertar el resultado en la tabla test_log
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    WHEN p_tipo_oper = 'UPDATE' THEN
        RSP := 'OK';
        UPDATE REGLASMIEMBROMESA SET NREGLA = 10 WHERE NREGLA = p_num_regla;
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    WHEN p_tipo_oper = 'DELETE' THEN
        RSP := 'OK';
        DELETE FROM REGLASMIEMBROMESA WHERE NREGLA = p_num_regla;
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    END CASE;

    -- Tratamiento de excepciones
    EXCEPTION
        WHEN e_null_mesaId THEN
            RSP := 'ERROR: Identificador de mesa nulo';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_n_regla THEN
            RSP := 'ERROR: Número de regla nulo';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_no_pk_ref THEN
            RSP := 'Error: No existe la clave principal referenciada';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_fk THEN
            RSP := 'Error: Existe una clave foranea que referencia a REGLASMIEMBROMESA';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN DUP_VAL_ON_INDEX THEN
            RSP := 'ERROR: Ya existe una REGLASMIEMBROMESA con esta PK'; 
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN NO_DATA_FOUND THEN
            RSP := 'ERROR: No existen datos con los parametros dados';  
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN OTHERS THEN
            RSP := 'ERROR: ' || SUBSTR(SQLERRM(-SQLCODE),1,100); 
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);

END p_test_reglas_miembro_mesa;

-- Procedimiento para el test de la tabla OrganizacionPolitica

create or replace PROCEDURE p_test_org_politica (
    p_siglas IN ORGANIZACIONPOLITICA.SIGLAS%TYPE,
    p_tipo IN ORGANIZACIONPOLITICA.TIPO%TYPE,
    p_denominacion IN ORGANIZACIONPOLITICA.DENOMINACION%TYPE,
    p_web IN ORGANIZACIONPOLITICA.WEB%TYPE,
    p_amb_act IN ORGANIZACIONPOLITICA.AMBITOACTUACION%TYPE,
    p_fines IN ORGANIZACIONPOLITICA.FINES%TYPE,
    p_tabla_mod IN VARCHAR2,
    p_tipo_oper IN VARCHAR2,
    RSP OUT VARCHAR2)
AS
    v_proced TEST_LOG.PROCEDIMIENTO%TYPE := 'p_test_org_politica';

    e_null_siglas EXCEPTION;
    e_null_tipo EXCEPTION;
    e_null_denom EXCEPTION;
    e_enum_tipo EXCEPTION;
    e_null_web EXCEPTION;
    e_null_amb_act EXCEPTION;
    e_null_fines EXCEPTION;
    e_no_pk_ref EXCEPTION;
    pragma exception_init(e_no_pk_ref, -2291);
    e_fk EXCEPTION;
    pragma exception_init(e_fk, -2292);

BEGIN
    -- Inicio del procedimiento 

    -- Comprobación de parámetros
    IF  (p_siglas IS NULL OR p_siglas ='') THEN
        RAISE e_null_siglas;
    ELSIF (p_tipo IS NULL OR p_tipo ='') THEN
        RAISE e_null_tipo;
    ELSIF (p_denominacion IS NULL OR p_denominacion ='') THEN
        RAISE e_null_denom;
    ELSIF (p_tipo NOT IN ('Partido Politico', 'Federacion','Agrupacion', 'Coalicion')) THEN
        RAISE e_enum_tipo;
    ELSIF  (p_web IS NULL OR p_web ='') THEN
        RAISE e_null_web;
    ELSIF  (p_amb_act IS NULL OR p_amb_act ='') THEN
        RAISE e_null_amb_act;
    ELSIF  (p_fines IS NULL OR p_fines ='') THEN
        RAISE e_null_fines;
    END IF;

    -- Asignación del valor OK a la variable de salida
    -- Selección de la operación a realizar 
    CASE 
    WHEN p_tipo_oper = 'INSERT' THEN
        -- Inserción en la tabla.
        INSERT INTO ORGANIZACIONPOLITICA VALUES 
            (p_siglas, p_tipo, p_denominacion, p_web, p_amb_act, p_fines);
        RSP := 'OK';
        -- Insertar el resultado en la tabla test_log
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    WHEN p_tipo_oper = 'UPDATE' THEN
        RSP := 'OK';
        UPDATE ORGANIZACIONPOLITICA SET TIPO = 'Federacion' WHERE SIGLAS = p_siglas;
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    WHEN p_tipo_oper = 'DELETE' THEN
        RSP := 'OK';
        DELETE FROM ORGANIZACIONPOLITICA WHERE SIGLAS = p_siglas;
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    END CASE;

    -- Tratamiento de excepciones
    EXCEPTION
        WHEN e_null_siglas THEN
            RSP := 'ERROR: Siglas de la organización nulas';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_tipo THEN
            RSP := 'ERROR: Tipo de org política nula';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_denom THEN
            RSP := 'ERROR: Denominación nula';  
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
             VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_enum_tipo THEN
            RSP := 'ERROR: Tipo de org. política no válido';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_web THEN
            RSP := 'ERROR: Web de la org. política nula';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
         WHEN e_null_amb_act THEN
            RSP := 'ERROR: Ambito de actuación nulo';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_fines THEN
            RSP := 'ERROR: Fines nulos';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_no_pk_ref THEN
            RSP := 'Error: No existe la clave principal referenciada';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_fk THEN
            RSP := 'Error: Existe una clave foranea que referencia a ORGANIZACIONPOLITICA';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN DUP_VAL_ON_INDEX THEN
            RSP := 'ERROR: Ya existe una ORGANIZACIONPOLITICA con esta PK';  
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN NO_DATA_FOUND THEN
            RSP := 'ERROR: No existen datos con los parametros dados';  
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN OTHERS THEN
            RSP := 'ERROR: ' || SUBSTR(SQLERRM(-SQLCODE),1,100); 
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);

END p_test_org_politica;


-- Procedimiento para el test de la tabla Presentacion

create or replace PROCEDURE p_test_presentacion (
    p_dni IN PRESENTACION.DNI%TYPE, 
    p_fec_cierre IN PRESENTACION.FECHACIERRE%TYPE, 
    p_siglas IN PRESENTACION.SIGLAS%TYPE,
    p_ambito IN PRESENTACION.AMBITO%TYPE,
    p_munic IN PRESENTACION.MUNICIPIO%TYPE, 
    p_fec_ini_cand IN PRESENTACION.FECHAINICANDIDATURA%TYPE,
    p_fec_cand IN PRESENTACION.FECHACANDIDATURA%TYPE,
    p_tabla_mod IN VARCHAR2,
    p_tipo_oper IN VARCHAR2,
    RSP OUT VARCHAR2)
AS
    v_proced TEST_LOG.PROCEDIMIENTO%TYPE := 'p_test_presentacion';

    e_null_dni EXCEPTION;
    e_null_fec_cierre EXCEPTION;
    e_null_siglas EXCEPTION;
    e_null_ambito EXCEPTION;
    e_null_munic EXCEPTION;
    e_null_fec_ini_cand EXCEPTION;
    e_plazo_cand EXCEPTION;
    e_no_pk_ref EXCEPTION;
    pragma exception_init(e_no_pk_ref, -2291);
    e_fk EXCEPTION;
    pragma exception_init(e_fk, -2292);

BEGIN
    -- Inicio del procedimiento 

    -- Comprobación de parámetros
    IF  (p_dni IS NULL OR p_dni ='') THEN
        RAISE e_null_dni;
    ELSIF (p_fec_cierre IS NULL OR p_fec_cierre ='') THEN
        RAISE e_null_fec_cierre;
    ELSIF (p_siglas IS NULL OR p_siglas ='') THEN
        RAISE e_null_siglas;
    ELSIF (p_ambito IS NULL OR p_ambito ='') THEN
        RAISE e_null_ambito;
    ELSIF  (p_munic IS NULL OR p_munic ='') THEN
        RAISE e_null_munic;
    ELSIF  (p_fec_ini_cand IS NULL OR p_fec_ini_cand ='') THEN
        RAISE e_null_fec_ini_cand;
    ELSIF  (p_fec_cand NOT BETWEEN p_fec_ini_cand AND p_fec_ini_cand + 5) THEN
        RAISE e_plazo_cand;
    END IF;

    -- Asignación del valor OK a la variable de salida
    -- Selección de la operación a realizar 
    CASE 
    WHEN p_tipo_oper = 'INSERT' THEN
        -- Inserción en la tabla.
        INSERT INTO PRESENTACION (DNI, FECHACIERRE, SIGLAS, AMBITO, MUNICIPIO, FECHAINICANDIDATURA, FECHACANDIDATURA) VALUES 
            (p_dni, p_fec_cierre, p_siglas,p_ambito, p_munic, p_fec_ini_cand, p_fec_cand);
        RSP := 'OK';
        -- Insertar el resultado en la tabla test_log
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    WHEN p_tipo_oper = 'UPDATE' THEN
        RSP := 'OK';
        UPDATE PRESENTACION SET SIGLAS = 'PDE' WHERE DNI = p_dni AND AMBITO = p_ambito;
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    WHEN p_tipo_oper = 'DELETE' THEN
        RSP := 'OK';
        DELETE FROM PRESENTACION WHERE DNI = p_dni AND AMBITO = p_ambito;
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    END CASE;

    -- Tratamiento de excepciones
    EXCEPTION
        WHEN e_null_dni THEN
            RSP := 'ERROR: DNI nulo';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_fec_cierre THEN
            RSP := 'ERROR: FECHACIERRE nula';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_siglas THEN
            RSP := 'ERROR: Siglas org. política nulas';  
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
             VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_ambito THEN
            RSP := 'ERROR: Ambito de elecciones nulo';  
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
             VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_munic THEN
            RSP := 'ERROR: MUNICIPIO de presentación';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
         WHEN e_null_fec_ini_cand THEN
            RSP := 'ERROR: Fecha de inicio de la candidatura nula';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_plazo_cand THEN
            RSP := 'ERROR: Fecha de presentación fuera de plazo';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_no_pk_ref THEN
            RSP := 'Error: No existe la clave principal referenciada';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_fk THEN
            RSP := 'Error: Existe una clave foranea que referencia a PRESENTACION';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN DUP_VAL_ON_INDEX THEN
            RSP := 'ERROR: Ya existe una PRESENTACION con esta PK';  
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN NO_DATA_FOUND THEN
            RSP := 'ERROR: No existen datos con los parametros dados';  
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN OTHERS THEN
            RSP := 'ERROR: ' || SUBSTR(SQLERRM(-SQLCODE),1,100); 
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);

END p_test_presentacion;

-- Procedimiento para el test de la tabla Informada

create or replace PROCEDURE p_test_informada (
    p_dni IN INFORMADA.DNI%TYPE, 
    p_fec_cierre IN INFORMADA.FECHACIERRE%TYPE, 
    p_fec_com_per IN INFORMADA.FECHACOMPERSONAL%TYPE, 
    p_info_voto_elec IN INFORMADA.VOTOELECTRINFO%TYPE, 
    p_mesaId IN INFORMADA.MESAID%TYPE,
    p_tabla_mod IN VARCHAR2,
    p_tipo_oper IN VARCHAR2,
    RSP OUT VARCHAR2)
AS
    v_proced TEST_LOG.PROCEDIMIENTO%TYPE := 'p_test_informada';

    e_null_dni EXCEPTION;
    e_null_fec_cierre EXCEPTION;
    e_null_fec_com EXCEPTION;
    e_no_pk_ref EXCEPTION;
    pragma exception_init(e_no_pk_ref, -2291);
    e_fk EXCEPTION;
    pragma exception_init(e_fk, -2292);

BEGIN
    -- Inicio del procedimiento 

    -- Comprobación de parámetros
    IF  (p_dni IS NULL OR p_dni ='') THEN
        RAISE e_null_dni;
    ELSIF (p_fec_cierre IS NULL OR p_fec_cierre ='') THEN
        RAISE e_null_fec_cierre;
    ELSIF (p_fec_com_per IS NULL OR p_fec_com_per ='') THEN
        RAISE e_null_fec_com;
    END IF;

    -- Asignación del valor OK a la variable de salida
    -- Selección de la operación a realizar 
    CASE 
    WHEN p_tipo_oper = 'INSERT' THEN
        -- Inserción en la tabla.
        INSERT INTO INFORMADA VALUES 
            (p_dni, p_fec_cierre, p_fec_com_per, p_info_voto_elec, p_mesaId);
        RSP := 'OK';
        -- Insertar el resultado en la tabla test_log
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    WHEN p_tipo_oper = 'UPDATE' THEN
        RSP := 'OK';
        UPDATE INFORMADA SET VOTOELECTRINFO = 'Entra y vota' WHERE DNI = p_dni AND FECHACIERRE = p_fec_cierre;
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    WHEN p_tipo_oper = 'DELETE' THEN
        RSP := 'OK';
        DELETE FROM INFORMADA WHERE DNI = p_dni AND FECHACIERRE = p_fec_cierre;
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    END CASE;

    -- Tratamiento de excepciones
    EXCEPTION
        WHEN e_null_dni THEN
            RSP := 'ERROR: DNI nulo';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_fec_cierre THEN
            RSP := 'ERROR: FECHACIERRE nula';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_fec_com THEN
            RSP := 'ERROR: Siglas org. política nulas';  
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
             VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_no_pk_ref THEN
            RSP := 'Error: No existe la clave principal referenciada';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_fk THEN
            RSP := 'Error: Existe una clave foranea que referencia a INFORMADA';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN DUP_VAL_ON_INDEX THEN
            RSP := 'ERROR: Ya existe una INFORMADA con esta PK';  
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN NO_DATA_FOUND THEN
            RSP := 'ERROR: No existen datos con los parametros dados';  
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN OTHERS THEN
            RSP := 'ERROR: ' || SUBSTR(SQLERRM(-SQLCODE),1,100); 
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);

END p_test_informada;

-- Procedimiento para el test de la tabla Vota

create or replace PROCEDURE p_test_vota (
    p_dni IN VOTA.DNI%TYPE,
    p_fec_cierre IN VOTA.FECHACIERRE%TYPE,
    p_mesaId IN VOTA.MESAID%TYPE,
    p_lugar_voto IN VOTA.LUGARVOTO%TYPE,
    p_hora_ini_voto IN VOTA.HORAINIVOTO%TYPE,
    p_hora_voto IN VOTA.HORAVOTO%TYPE,
    p_tabla_mod IN VARCHAR2,
    p_tipo_oper IN VARCHAR2,
    RSP OUT VARCHAR2)
AS
    v_proced TEST_LOG.PROCEDIMIENTO%TYPE := 'p_test_vota';

    e_null_dni EXCEPTION;
    e_null_fecha_cierre EXCEPTION;
    e_null_mesaId EXCEPTION;
    e_null_lug_voto EXCEPTION;
    e_null_hIni_voto EXCEPTION;
    e_null_hora_voto EXCEPTION;
    e_plazo_vota EXCEPTION;
    e_enum_lugar_voto EXCEPTION;
    e_no_pk_ref EXCEPTION;
    pragma exception_init(e_no_pk_ref, -2291);
    e_fk EXCEPTION;
    pragma exception_init(e_fk, -2292);

BEGIN
    -- Inicio del procedimiento

    -- Comprobación de parámetros lugarVoto IN ('Presencial', 'Electronico','Correspondencia')
    IF (p_dni IS NULL OR p_dni ='') THEN
        RAISE e_null_dni;
    ELSIF (p_fec_cierre IS NULL OR p_fec_cierre ='') THEN
        RAISE e_null_fecha_cierre;
    ELSIF (p_mesaId IS NULL OR p_mesaId ='') THEN
        RAISE e_null_mesaId; 
    ELSIF (p_lugar_voto IS NULL OR p_lugar_voto ='') THEN
        RAISE e_null_lug_voto;
    ELSIF (p_hora_ini_voto IS NULL OR p_hora_ini_voto ='') THEN
        RAISE e_null_hIni_voto;
    ELSIF (p_hora_voto IS NULL OR p_hora_voto ='') THEN
        RAISE e_null_hora_voto;        
    ELSIF (p_hora_voto NOT BETWEEN p_hora_ini_voto AND p_hora_ini_voto + INTERVAL '11' HOUR) THEN
        RAISE e_plazo_vota;
    ELSIF (p_lugar_voto NOT IN ('Presencial', 'Electronico','Correspondencia')) THEN
        RAISE e_enum_lugar_voto;
    END IF;

    -- Asignación del valor OK a la variable de salida
    -- Selección de la operación a realizar 
    CASE 
    WHEN p_tipo_oper = 'INSERT' THEN
        -- Inserción en la tabla.
        INSERT INTO VOTA
        VALUES (p_dni, p_fec_cierre, p_mesaId, p_lugar_voto, p_hora_ini_voto, p_hora_voto);
        RSP := 'OK';
        -- Insertar el resultado en la tabla test_log
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    WHEN p_tipo_oper = 'UPDATE' THEN
        RSP := 'OK';
        UPDATE VOTA SET LUGARVOTO = 'Electronico' WHERE MESAID = p_mesaId AND DNI = p_dni;
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    WHEN p_tipo_oper = 'DELETE' THEN
        RSP := 'OK';
        DELETE FROM VOTA WHERE MESAID = p_mesaId AND DNI = p_dni;
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    END CASE;

    -- Tratamiento de excepciones
    EXCEPTION
         WHEN e_null_dni THEN
            RSP := 'ERROR: DNI nulo';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
         WHEN e_null_fecha_cierre THEN
            RSP := 'ERROR: Fecha de cierre del censo nula';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_mesaId THEN
            RSP := 'ERROR: Identificador de mesa nulo';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_lug_voto THEN
            RSP := 'ERROR: Lugar de voto nulo';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_hIni_voto THEN
            RSP := 'ERROR: Hora de inicio de voto nula';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_hora_voto THEN
            RSP := 'ERROR: Hora de voto nula';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_plazo_vota THEN
            RSP := 'ERROR: La votación es entre las 9 y las 20 horas';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_enum_lugar_voto THEN
            RSP := 'ERROR: El lugar de voto no es válido';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_no_pk_ref THEN
            RSP := 'Error: No existe la clave principal referenciada';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_fk THEN
            RSP := 'Error: Existe una clave foranea que referencia a VOTA';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN DUP_VAL_ON_INDEX THEN
            RSP := 'ERROR: Ya existe una VOTA con esta PK';  
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN NO_DATA_FOUND THEN
            RSP := 'ERROR: No existen datos con los parametros dados';  
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN OTHERS THEN
            RSP := 'ERROR: ' || SUBSTR(SQLERRM(-SQLCODE),1,100); 
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);

END p_test_vota;

-- Procedimiento para el test de la tabla RecibeVoto

create or replace PROCEDURE p_test_recibevoto (
    p_siglas IN RECIBEVOTO.SIGLAS%TYPE,
    p_mesaId IN RECIBEVOTO.MESAID%TYPE,
    p_hora_ini_rec IN RECIBEVOTO.HORAINIRECUENTO%TYPE,
    p_hora_rec IN RECIBEVOTO.HORARECUENTO%TYPE,
    p_es_nulo IN RECIBEVOTO.ESNULO%TYPE,
    p_es_blanco IN RECIBEVOTO.ESBLANCO%TYPE,
    p_tabla_mod IN VARCHAR2,
    p_tipo_oper IN VARCHAR2,
    RSP OUT VARCHAR2)
AS
    v_proced TEST_LOG.PROCEDIMIENTO%TYPE := 'p_test_recibevoto';

    e_null_mesaId EXCEPTION;
    e_null_hora_in_rec EXCEPTION;
    e_null_hora_rec EXCEPTION;
    e_null_es_nulo EXCEPTION;
    e_null_es_blanco EXCEPTION;
    e_enum_esNulo EXCEPTION;
    e_enum_esBlanco EXCEPTION;
    e_plazo_recuento EXCEPTION;
    e_no_pk_ref EXCEPTION;
    pragma exception_init(e_no_pk_ref, -2291);
    e_fk EXCEPTION;
    pragma exception_init(e_fk, -2292);

BEGIN
    -- Inicio del procedimiento

    -- Comprobación de parámetros HORARECUENTO IN ('Presencial', 'Electronico','Correspondencia')
    IF (p_mesaId IS NULL OR p_mesaId ='') THEN
        RAISE e_null_mesaId;
    ELSIF (p_hora_ini_rec IS NULL OR p_hora_ini_rec ='') THEN
        RAISE e_null_hora_in_rec; 
    ELSIF (p_hora_rec IS NULL OR p_hora_rec ='') THEN
        RAISE e_null_hora_rec;
    ELSIF (p_es_nulo IS NULL OR p_es_nulo ='') THEN
        RAISE e_null_es_nulo;
    ELSIF (p_es_blanco IS NULL OR p_es_blanco ='') THEN
        RAISE e_null_es_blanco;        
    ELSIF (p_es_nulo NOT IN (0,1)) THEN
        RAISE e_enum_esNulo;
    ELSIF (p_es_blanco NOT IN (0,1)) THEN
        RAISE e_enum_esBlanco;
    ELSIF (p_hora_rec NOT BETWEEN p_hora_ini_rec AND p_hora_ini_rec + INTERVAL '40' HOUR) THEN
        RAISE e_plazo_recuento;
    END IF;

    -- Asignación del valor OK a la variable de salida
    -- Selección de la operación a realizar 
    CASE 
    WHEN p_tipo_oper = 'INSERT' THEN
        -- Inserción en la tabla.
        INSERT INTO RECIBEVOTO
        VALUES (p_siglas, p_mesaId, p_hora_ini_rec, p_hora_rec, p_es_nulo, p_es_blanco);
        RSP := 'OK';
        -- Insertar el resultado en la tabla test_log
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    WHEN p_tipo_oper = 'UPDATE' THEN
        RSP := 'OK';
        UPDATE RECIBEVOTO SET HORARECUENTO = 'Electronico' WHERE MESAID = p_mesaId AND HORARECUENTO = p_hora_rec;
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    WHEN p_tipo_oper = 'DELETE' THEN
        RSP := 'OK';
        DELETE FROM RECIBEVOTO WHERE MESAID = p_mesaId AND HORARECUENTO = p_hora_rec;
        INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
        VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
    END CASE;

    -- Tratamiento de excepciones
    EXCEPTION
        WHEN e_null_mesaId THEN
            RSP := 'ERROR: Identificador de mesa nulo';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
         WHEN e_null_hora_in_rec THEN
            RSP := 'ERROR: Hora de inicio de recuento nula';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_hora_rec THEN
            RSP := 'ERROR: Hora de recuento nula';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_es_nulo THEN
            RSP := 'ERROR: El voto debe ser nulo o válido';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_null_es_blanco THEN
            RSP := 'ERROR: El voto debe ser en blanco o no';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_enum_esNulo THEN
            RSP := 'ERROR: El voto nulo tiene valores 0 ó 1';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_enum_esBlanco THEN
            RSP := 'ERROR: El voto en blanco tiene valores 0 ó 1';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_plazo_recuento THEN
            RSP := 'ERROR: El recuento se produce fuera de plazo';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_no_pk_ref THEN
            RSP := 'Error: No existe la clave principal referenciada';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN e_fk THEN
            RSP := 'Error: Existe una clave foranea que referencia a RECIBEVOTO';
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN DUP_VAL_ON_INDEX THEN
            RSP := 'ERROR: Ya existe una RECIBEVOTO con esta PK';  
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN NO_DATA_FOUND THEN
            RSP := 'ERROR: No existen datos con los parametros dados';  
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);
        WHEN OTHERS THEN
            RSP := 'ERROR: ' || SUBSTR(SQLERRM(-SQLCODE),1,100); 
            INSERT INTO TEST_LOG(procedimiento, tabla_test, operacion, resultado)
            VALUES(v_proced, p_tabla_mod, p_tipo_oper, RSP);

END p_test_recibevoto;