
-- Tabla para reflejar los cambios en las tablas de la base de datos.

CREATE TABLE MANTENIMIENTO_LOG (
log_id INTEGER CONSTRAINT PK_Mant_Log PRIMARY KEY,
user_id VARCHAR2(30 CHAR),
fecha_operac TIMESTAMP,
trigger_activado VARCHAR2(100 CHAR),
tabla_afectada VARCHAR2 (100 CHAR),
tipo_operac VARCHAR2(10 CHAR),
resultado VARCHAR2 (100 CHAR),
sesion_id VARCHAR2(20 CHAR)
);

-- Secuencia para identificar cada fila de la tabla mantenimiento_log
CREATE SEQUENCE seq_mantenimiento_log INCREMENT BY 1 START WITH 1;


-- Disparador para insertar el valor autoincrementado de la secuencia
CREATE OR REPLACE TRIGGER tr_mant_log_id 
   BEFORE INSERT ON MANTENIMIENTO_LOG
   FOR EACH ROW
BEGIN
   SELECT seq_mantenimiento_log.nextval INTO :NEW.log_id FROM DUAL;
END;

-- Disparador para insertar en la tabla la identificación de la sesión
CREATE OR REPLACE TRIGGER tr_mant_log_session
	BEFORE INSERT ON MANTENIMIENTO_LOG
	FOR EACH ROW
BEGIN
	SELECT substr(userenv('SESSIONID'),1,10) INTO :NEW.sesion_id FROM DUAL;
END;

-- Disparador para insertar en la tabla la fecha y hora en la que se
-- realiza la operación.
CREATE OR REPLACE TRIGGER tr_mant_log_date
	BEFORE INSERT ON MANTENIMIENTO_LOG
	FOR EACH ROW
BEGIN
	SELECT LOCALTIMESTAMP INTO :NEW.fecha_operac FROM DUAL;
END;


-- Disparador para insertar en la tabla el usuario que realiza la operación.
CREATE OR REPLACE TRIGGER tr_mant_log_user
	BEFORE INSERT ON MANTENIMIENTO_LOG
	FOR EACH ROW
BEGIN
	SELECT USER_ID INTO :NEW.user_id FROM user_users;
END;
