

CREATE TABLE Convocatoria (
	fechaConvBOE DATE CONSTRAINT NN_FechaCom NOT NULL,
	fechaElecciones DATE CONSTRAINT NN_ConvFE NOT NULL, 
	ambito VARCHAR2 (20 CHAR) CONSTRAINT NN_ReglaCalc NOT NULL,
	fechaCenso DATE,
	reglaCalcResultado VARCHAR2 (20 CHAR),
	CONSTRAINT CH_Ambito CHECK (ambito IN ('Generales', 'Municipales', 'Parlamento Europeo', 'Autonomicas')),
	CONSTRAINT CH_ReglCalcRes CHECK (reglaCalcResultado IN ('D’Hondt', 'Droop', 'Hagenbach-Bischoff', 'Resto Mayor', 'Sainte-Laguë')),
	CONSTRAINT PK_Comunicacion PRIMARY KEY (fechaConvBOE, ambito),
	CONSTRAINT CH_FechaElec CHECK (fechaElecciones >= fechaConvBOE + 45) 
);


CREATE TABLE ReglasElecciones (
	fechaConvBOE DATE,
	ambito VARCHAR (20 CHAR),
	nRegla INTEGER,
	regla VARCHAR2 (500 CHAR),
	CONSTRAINT PK_ReglasElec PRIMARY KEY (fechaConvBOE, ambito, nRegla),
	CONSTRAINT FK_ReglasElec FOREIGN KEY (fechaConvBOE, ambito) 
				REFERENCES Convocatoria (fechaConvBOE, ambito)
);


CREATE TABLE Demarcacion (
	provincia VARCHAR2 (20 CHAR) CONSTRAINT PK_Demarcacion PRIMARY KEY,
	region VARCHAR2 (20 CHAR) CONSTRAINT NN_Region NOT NULL,
	pais VARCHAR2 (20 CHAR) CONSTRAINT NN_Pais NOT NULL
);

CREATE TABLE Recibe (
	fechaConvBOE DATE CONSTRAINT NN_FechaElec NOT NULL, 
	ambito VARCHAR (20 CHAR),
	provincia VARCHAR (20 CHAR),
	numEscanos INTEGER CONSTRAINT NN_NumEsc NOT NULL,
	CONSTRAINT PK_Recibe PRIMARY KEY (fechaConvBOE, ambito, provincia),
	CONSTRAINT FK_RecibeConv FOREIGN KEY (fechaConvBOE, ambito) 
				REFERENCES Convocatoria (fechaConvBOE, ambito),
	CONSTRAINT FK_RecibeDemar FOREIGN KEY (provincia) REFERENCES Demarcacion (provincia),
	CONSTRAINT CH_RecibeNumEsc CHECK (numEscanos > 0)
);


CREATE TABLE CentroVotacion (
	municipio VARCHAR (30 CHAR) CONSTRAINT NN_Mun NOT NULL,
	provincia VARCHAR (20 CHAR),
	CONSTRAINT PK_CentroVotacion PRIMARY KEY (municipio, provincia),
	CONSTRAINT FK_CentroVot FOREIGN KEY (provincia) REFERENCES Demarcacion (provincia)
	
);

CREATE TABLE ComunicacionPersonal(
	fechaComPersonal DATE CONSTRAINT PK_ComPers PRIMARY KEY
);

CREATE TABLE CensoElectoral(
	fechaCierre DATE CONSTRAINT PK_CensoElec PRIMARY KEY
);

CREATE TABLE MesaElectoral (
	mesaId VARCHAR (20 CHAR)
		CONSTRAINT AK_MesaId UNIQUE
		CONSTRAINT NN_MesaId NOT NULL,
	fechaProceso DATE,
	ambito VARCHAR (20 CHAR),
	codigoPostal VARCHAR (5 CHAR),
	calle VARCHAR (30 CHAR),
	numero INTEGER,
	municipio VARCHAR (30 CHAR),
	demarcacion VARCHAR (20 CHAR),
	numVotantes INTEGER DEFAULT (0),
	numVotos INTEGER DEFAULT (0),
	CONSTRAINT PK_MesaElect PRIMARY KEY (fechaProceso, ambito, codigoPostal, calle, numero, municipio, demarcacion),
	CONSTRAINT FK_MesaMun FOREIGN KEY (municipio, demarcacion) REFERENCES CentroVotacion (municipio, provincia),
	CONSTRAINT CH_NumVotantes CHECK (numeroVotantes <= 2000),
	CONSTRAINT CH_NumVotos CHECK (numeroVotos <= numeroVotantes) 

);


CREATE TABLE Persona (
	DNI VARCHAR (20 CHAR) CONSTRAINT NN_DNI NOT NULL,
	fechaCierre DATE CONSTRAINT NN_fechaCierre NOT NULL,
	nombre VARCHAR (20 CHAR) CONSTRAINT NN_Nombre NOT NULL,
	apellidos VARCHAR (40 CHAR) CONSTRAINT NN_Apell NOT NULL,
	sexo VARCHAR (20 CHAR) CONSTRAINT NN_Sexo NOT NULL,
	nacionalidad VARCHAR (20 CHAR) CONSTRAINT NN_Nacio NOT NULL,
	fechaNacimiento DATE CONSTRAINT NN_FechaNac NOT NULL,
	lugarNacimiento VARCHAR (40 CHAR),
	estudios VARCHAR (40 CHAR) CONSTRAINT NN_Estud NOT NULL,
	telefono VARCHAR (20 CHAR),
	procesado VARCHAR (2 CHAR) CONSTRAINT NN_Procesado NOT NULL,
	direccion VARCHAR (50 CHAR),
	CONSTRAINT PK_Persona PRIMARY KEY (DNI, fechaCierre),
	CONSTRAINT FK_Persona FOREIGN KEY (fechaCierre) REFERENCES CensoElectoral (fechaCierre),
	CONSTRAINT CH_Procesado CHECK (procesado IN ('SI','NO')),
	CONSTRAINT CH_Sexo CHECK (sexo IN ('Hombre', 'Mujer')),
	CONSTRAINT CH_Estudios CHECK (estudios IN ('ESO', 'Bachillerato', 'FP', 'Universitarios'))

);

CREATE TABLE Candidato (
	DNI VARCHAR (20 CHAR) CONSTRAINT NN_CandDNI NOT NULL,
	fechaCierre DATE CONSTRAINT NN_CandFechaCierre NOT NULL,
	esSuplente  SMALLINT DEFAULT (0) CONSTRAINT NN_ESSUPL NOT NULL;
	CONSTRAINT PK_Candidato PRIMARY KEY (DNI, fechaCierre),
	CONSTRAINT CH_esSuplente CHECK (esSuplente IN (0,1)),
	CONSTRAINT FK_Candidato FOREIGN KEY (DNI, fechaCierre) REFERENCES Persona (DNI, fechaCierre)
);

CREATE TABLE Presidente (
	DNI VARCHAR (20 CHAR) CONSTRAINT NN_PresDNI NOT NULL,
	fechaCierre DATE CONSTRAINT NN_PresFechaCierre NOT NULL, 
	numAlegaciones INTEGER DEFAULT (0) CONSTRAINT NN_PresNumAleg NOT NULL,
	numParticipaciones INTEGER DEFAULT (0) CONSTRAINT NN_PresNumPart NOT NULL,
	CONSTRAINT PK_Pres PRIMARY KEY (DNI, fechaCierre),
	CONSTRAINT FK_Pres FOREIGN KEY (DNI, fechaCierre) REFERENCES Persona (DNI, fechaCierre)
);

CREATE TABLE Vocal (
	DNI VARCHAR (20 CHAR) CONSTRAINT NN_VocalDNI NOT NULL,
	fechaCierre DATE CONSTRAINT NN_VocalFechaCierre NOT NULL,
	numAlegaciones INTEGER DEFAULT (0) CONSTRAINT NN_VocalNumAleg NOT NULL,
	numParticipaciones INTEGER DEFAULT (0) CONSTRAINT NN_VacalNumPart NOT NULL,
	CONSTRAINT PK_Vocal PRIMARY KEY (DNI, fechaCierre),
	CONSTRAINT FK_Vocal FOREIGN KEY (DNI, fechaCierre) REFERENCES Persona (DNI, fechaCierre)
);
CREATE TABLE Suplente (
	DNI VARCHAR (20 CHAR) CONSTRAINT NN_SuplDNI NOT NULL,
	fechaCierre DATE CONSTRAINT NN_SuplFechaCierre NOT NULL, 
	numAlegaciones INTEGER DEFAULT (0) CONSTRAINT NN_SuplNumAleg NOT NULL,
	numParticipaciones INTEGER DEFAULT(0) CONSTRAINT NN_SuplNumPart NOT NULL,
	CONSTRAINT PK_Supl PRIMARY KEY (DNI, fechaCierre),
	CONSTRAINT FK_Supl FOREIGN KEY (DNI, fechaCierre) REFERENCES Persona (DNI, fechaCierre)
);

CREATE TABLE Compuesta (
	mesaId VARCHAR (20 CHAR) CONSTRAINT NN_CompMesaId NOT NULL,
	fechaCierre DATE CONSTRAINT NN_CompFechaCierre NOT NULL,
	DNI VARCHAR (20 CHAR) CONSTRAINT NN_CompDNI NOT NULL,
	fechaIniSorteo DATE CONSTRAINT NN_CompFechIncSort NOT NULL,
	fechaSorteo DATE,
	fechaIniNotif DATE,
	fechaNotif DATE,
	fechaIniAlegac DATE,
	fechaAlegacion DATE,
	CONSTRAINT PK_Compuesta PRIMARY KEY (mesaId, DNI, fechaCierre),
	CONSTRAINT FK_CompMesa FOREIGN KEY (mesaId) REFERENCES MesaElectoral (mesaId),
	CONSTRAINT FK_CompPers FOREIGN KEY (DNI, fechaCierre) REFERENCES Persona (DNI, fechaCierre),
	CONSTRAINT CH_ComFechaSort CHECK (fechaSorteo BETWEEN fechaIniSorteo AND fechaIniSorteo + 4),
	CONSTRAINT CH_CompFechIniNot CHECK (fechaIniNotif = fechaSorteo),
	CONSTRAINT CH_ComFechaNotif CHECK (fechaNotif BETWEEN fechaIniNotif AND fechaIniNotif + 3),
	CONSTRAINT CH_CompFechIniAle CHECK (fechaIniAlegac = fechaNotif),
	CONSTRAINT CH_ComFechaAleg CHECK (fechaAlegacion BETWEEN fechaIniAlegac AND fechaIniAlegac + 7)
);


CREATE TABLE ReglasMiembroMesa (
	mesaId VARCHAR (20 CHAR),
	nRegla INTEGER,
	regla VARCHAR2 (500 CHAR),
	CONSTRAINT PK_ReglasMiemMesa PRIMARY KEY (mesaId, DNI, fechaCierre, nRegla),
	CONSTRAINT FK_RegMMPerson FOREIGN KEY (DNI, fechaCierre) REFERENCES Persona (DNI, fechaCierre),
	CONSTRAINT FK_REglasMiemMesa FOREIGN KEY (mesaId) REFERENCES MesaElectoral (mesaId)
);


CREATE TABLE OrganizacionPolitica (
	siglas VARCHAR (20 CHAR) CONSTRAINT PK_OrgPol PRIMARY KEY,
	tipo VARCHAR (20 CHAR) CONSTRAINT NN_Tipo NOT NULL,
	denominacion VARCHAR (40 CHAR) 
		CONSTRAINT NN_Denom NOT NULL
		CONSTRAINT U_Denom UNIQUE,
	web VARCHAR (20 CHAR) CONSTRAINT NN_Web NOT NULL,
	ambitoActuacion VARCHAR (30 CHAR) CONSTRAINT NN_AmbAct NOT NULL,
	fines VARCHAR (500 CHAR) CONSTRAINT NN_Fines NOT NULL,
	CONSTRAINT CH_tipo CHECK (tipo IN ('Partido Politico', 'Federacion','Agrupacion', 'Coalicion'))
);

CREATE TABLE Presentacion (
	DNI VARCHAR (20 CHAR),
	fechaCierre DATE,
	siglas VARCHAR (20 CHAR),
	ambito VARCHAR (20 CHAR) ,
	municipio VARCHAR (30 CHAR) CONSTRAINT NN_PresMun NOT NULL,
	fechaIniCandidatura DATE CONSTRAINT NN_FecIniCan NOT NULL, #Convocatoria + 15
	fechaCandidatura DATE,
	CONSTRAINT PK_Presentacion PRIMARY KEY (DNI, fechaCierre, siglas, ambito),
	CONSTRAINT FK_PresCand FOREIGN KEY (DNI, fechaCierre) REFERENCES Candidato (DNI, fechaCierre),
	CONSTRAINT FK_PresSig FOREIGN KEY (siglas) REFERENCES OrganizacionPolitica (siglas),
	CONSTRAINT CH_PresFechaCand CHECK (fechaCandidatura BETWEEN fechaIniCandidatura AND fechaIniCandidatura + 5)

);

CREATE TABLE Informada (
	DNI VARCHAR (20 CHAR),
	fechaCierre DATE, 
	fechaComPersonal DATE,
	votoElectrInfo VARCHAR (500 CHAR),
	mesaId VARCHAR (20 CHAR),
	CONSTRAINT PK_Informada PRIMARY KEY (DNI, fechaCierre, fechaComPersonal),
	CONSTRAINT FK_InformPers FOREIGN KEY (DNI, fechaCierre) REFERENCES Persona (DNI, fechaCierre),
	CONSTRAINT FK_InfComPer FOREIGN KEY (fechaComPersonal) REFERENCES ComunicacionPersonal (fechaComPersonal),
	CONSTRAINT FK_InformMesa FOREIGN KEY (mesaId) REFERENCES MesaElectoral (mesaId)
);

CREATE TABLE Vota (
	DNI VARCHAR (20 CHAR),
	fechaCierre DATE CONSTRAINT NN_VotaFechaCierre NOT NULL, 
	mesaId VARCHAR (20 CHAR),
	lugarVoto VARCHAR (20 CHAR) CONSTRAINT NN_lugarVoto NOT NULL,
	horaIniVoto TIMESTAMP CONSTRAINT NN_horaIniVoto NOT NULL,
	horaVoto TIMESTAMP CONSTRAINT NN_horaVoto NOT NULL,
	CONSTRAINT PK_Vota PRIMARY KEY (DNI, fechaCierre, mesaId),
	CONSTRAINT FK_VotaPers FOREIGN KEY (DNI, fechaCierre) REFERENCES Persona (DNI, fechaCierre),
	CONSTRAINT FK_VotaMesaId FOREIGN KEY (mesaId) REFERENCES MesaElectoral (mesaId),
	CONSTRAINT CH_lugarVoto CHECK (lugarVoto IN ('Presencial', 'Electronico','Correspondencia')),
	CONSTRAINT CH_HoraVoto CHECK (horaVoto BETWEEN horaIniVoto AND horaIniVoto + INTERVAL '11' HOUR)
	
);

CREATE TABLE RecibeVoto (
	siglas VARCHAR (20 CHAR),
	mesaId VARCHAR (20 CHAR),
	horaIniRecuento TIMESTAMP CONSTRAINT NN_HoraIniRecVoto NOT NULL,
	horaRecuento TIMESTAMP,
	esNulo SMALLINT DEFAULT (0) CONSTRAINT NN_EsNulo NOT NULL,
	esBlanco SMALLINT DEFAULT (0) CONSTRAINT NN_EsBlanco NOT NULL,
	CONSTRAINT PK_RecibeVoto PRIMARY KEY (horaRecuento, mesaId),
	CONSTRAINT CH_esNulo CHECK (esNulo IN (0,1)),
	CONSTRAINT CH_esBlanco CHECK (esBlanco IN (0,1)),
	CONSTRAINT FK_RecVotoMesaId FOREIGN KEY (mesaId) REFERENCES MesaElectoral (mesaId),
	CONSTRAINT CK_HoraRecVoto CHECK (horaRecuento BETWEEN horaIniRecuento AND horaIniRecuento + INTERVAL '40' HOUR)
);



CREATE TABLE TEMP_TABLE_RES(
   	SUM_VOTOS NUMBER, 
	SIGLAS VARCHAR2(20 CHAR), 
	DEMARCACION VARCHAR2(20 CHAR), 
	NUM_ESC NUMBER(*,0) DEFAULT (0),
	CONSTRAINT PK_TTR PRIMARY KEY (SIGLAS, DEMARCACION)
);

CREATE TABLE h_Elecciones(
   	fecha_elecciones DATE, 
	ambito VARCHAR2(20 CHAR), 
	proc_mas_martic VARCHAR2(30 CHAR), 
	munic_mas_martic VARCHAR2(30 CHAR), 
	porc_elect_year NUMBER, 
	porc_dif_voto_ant NUMBER, 
	num_mas_dos NUMBER DEFAULT (0), 
	porc_reclam NUMBER DEFAULT (0), 
	num_todas10 NUMBER DEFAULT (0), 
	porc_sin_sorteo NUMBER,
	top_20_Mesas VARCHAR (300 CHAR),
	mesaId VARCHAR (20 CHAR),
	porc_sin_localizar NUMBER,
	proc_mas_candid VARCHAR2(30 CHAR), 
	idResultado VARCHAR (20 CHAR),
	CONSTRAINT NN_h_ElcIdMesa20 NOT NULL,
	CONSTRAINT PK_HELEC PRIMARY KEY (fecha_elecciones, ambito),
	CONSTRAINT FK_H_ELECCIONES FOREIGN KEY (mesaId) REFERENCES MESAELECTORAL (mesaId)
);

